@@ -5,7 +5,7 @@
  *
  * The MIT License
  *
- * @copyright Copyright (c) 2017-2023 TileDB Inc.
+ * @copyright Copyright (c) 2017-2024 TileDB Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -41,7 +41,6 @@
 #include "tiledb/sm/c_api/tiledb.h"
 #include "tiledb/sm/c_api/tiledb_serialization.h"
 #include "tiledb/sm/enums/serialization_type.h"
-#include "tiledb/sm/misc/utils.h"
 
 #include <iostream>
 #include <sstream>
@@ -424,11 +423,16 @@ void SparseHeterFx::check_est_result_size_float_int64(
   tiledb_query_t* query;
   rc = tiledb_query_alloc(ctx_, array, TILEDB_READ, &query);
   CHECK(rc == TILEDB_OK);
-  rc = tiledb_query_add_range(
-      ctx_, query, 0, &subarray_f[0], &subarray_f[1], nullptr);
+  tiledb_subarray_t* subarray;
+  rc = tiledb_subarray_alloc(ctx_, array, &subarray);
   CHECK(rc == TILEDB_OK);
-  rc = tiledb_query_add_range(
-      ctx_, query, 1, &subarray_i[0], &subarray_i[1], nullptr);
+  rc = tiledb_subarray_add_range(
+      ctx_, subarray, 0, &subarray_f[0], &subarray_f[1], nullptr);
+  CHECK(rc == TILEDB_OK);
+  rc = tiledb_subarray_add_range(
+      ctx_, subarray, 1, &subarray_i[0], &subarray_i[1], nullptr);
+  CHECK(rc == TILEDB_OK);
+  rc = tiledb_query_set_subarray_t(ctx_, query, subarray);
   CHECK(rc == TILEDB_OK);
 
   // Check error for zipped coordinates
@@ -452,6 +456,7 @@ void SparseHeterFx::check_est_result_size_float_int64(
   // Clean up
   tiledb_array_free(&array);
   tiledb_query_free(&query);
+  tiledb_subarray_free(&subarray);
 }
 
 void SparseHeterFx::check_est_result_size_int64_float(
@@ -470,11 +475,16 @@ void SparseHeterFx::check_est_result_size_int64_float(
   tiledb_query_t* query;
   rc = tiledb_query_alloc(ctx_, array, TILEDB_READ, &query);
   CHECK(rc == TILEDB_OK);
-  rc = tiledb_query_add_range(
-      ctx_, query, 0, &subarray_i[0], &subarray_i[1], nullptr);
+  tiledb_subarray_t* subarray;
+  rc = tiledb_subarray_alloc(ctx_, array, &subarray);
   CHECK(rc == TILEDB_OK);
-  rc = tiledb_query_add_range(
-      ctx_, query, 1, &subarray_f[0], &subarray_f[1], nullptr);
+  rc = tiledb_subarray_add_range(
+      ctx_, subarray, 0, &subarray_i[0], &subarray_i[1], nullptr);
+  CHECK(rc == TILEDB_OK);
+  rc = tiledb_subarray_add_range(
+      ctx_, subarray, 1, &subarray_f[0], &subarray_f[1], nullptr);
+  CHECK(rc == TILEDB_OK);
+  rc = tiledb_query_set_subarray_t(ctx_, query, subarray);
   CHECK(rc == TILEDB_OK);
 
   // Check error for zipped coordinates
@@ -498,6 +508,7 @@ void SparseHeterFx::check_est_result_size_int64_float(
   // Clean up
   tiledb_array_free(&array);
   tiledb_query_free(&query);
+  tiledb_subarray_free(&subarray);
 }
 
 void SparseHeterFx::write_sparse_array_float_int64(
@@ -631,12 +642,17 @@ void SparseHeterFx::check_read_sparse_array_float_int64(
   REQUIRE(rc == TILEDB_OK);
   rc = tiledb_query_set_layout(ctx_, query, layout);
   REQUIRE(rc == TILEDB_OK);
-  rc = tiledb_query_add_range(
-      ctx_, query, 0, &subarray_f[0], &subarray_f[1], nullptr);
+  tiledb_subarray_t* subarray;
+  rc = tiledb_subarray_alloc(ctx_, array, &subarray);
+  CHECK(rc == TILEDB_OK);
+  rc = tiledb_subarray_add_range(
+      ctx_, subarray, 0, &subarray_f[0], &subarray_f[1], nullptr);
   REQUIRE(rc == TILEDB_OK);
-  rc = tiledb_query_add_range(
-      ctx_, query, 1, &subarray_i[0], &subarray_i[1], nullptr);
+  rc = tiledb_subarray_add_range(
+      ctx_, subarray, 1, &subarray_i[0], &subarray_i[1], nullptr);
   REQUIRE(rc == TILEDB_OK);
+  rc = tiledb_query_set_subarray_t(ctx_, query, subarray);
+  CHECK(rc == TILEDB_OK);
   rc = tiledb_query_submit(ctx_, query);
   REQUIRE(rc == TILEDB_OK);
 
@@ -659,6 +675,7 @@ void SparseHeterFx::check_read_sparse_array_float_int64(
   // Clean up
   tiledb_array_free(&array);
   tiledb_query_free(&query);
+  tiledb_subarray_free(&subarray);
 }
 
 void SparseHeterFx::check_read_sparse_array_int64_float(
@@ -695,12 +712,17 @@ void SparseHeterFx::check_read_sparse_array_int64_float(
   REQUIRE(rc == TILEDB_OK);
   rc = tiledb_query_set_layout(ctx_, query, layout);
   REQUIRE(rc == TILEDB_OK);
-  rc = tiledb_query_add_range(
-      ctx_, query, 0, &subarray_i[0], &subarray_i[1], nullptr);
+  tiledb_subarray_t* subarray;
+  rc = tiledb_subarray_alloc(ctx_, array, &subarray);
+  CHECK(rc == TILEDB_OK);
+  rc = tiledb_subarray_add_range(
+      ctx_, subarray, 0, &subarray_i[0], &subarray_i[1], nullptr);
   REQUIRE(rc == TILEDB_OK);
-  rc = tiledb_query_add_range(
-      ctx_, query, 1, &subarray_f[0], &subarray_f[1], nullptr);
+  rc = tiledb_subarray_add_range(
+      ctx_, subarray, 1, &subarray_f[0], &subarray_f[1], nullptr);
   REQUIRE(rc == TILEDB_OK);
+  rc = tiledb_query_set_subarray_t(ctx_, query, subarray);
+  CHECK(rc == TILEDB_OK);
   rc = tiledb_query_submit(ctx_, query);
   REQUIRE(rc == TILEDB_OK);
 
@@ -723,6 +745,7 @@ void SparseHeterFx::check_read_sparse_array_int64_float(
   // Clean up
   tiledb_array_free(&array);
   tiledb_query_free(&query);
+  tiledb_subarray_free(&subarray);
 }
 
 TEST_CASE_METHOD(
@@ -979,7 +1002,15 @@ TEST_CASE_METHOD(
   REQUIRE(error == nullptr);
 
   auto rc = tiledb_config_set(
-      config, "sm.consolidation.total_buffer_size", "1048576", &error);
+      config, "sm.mem.consolidation.buffers_weight", "1", &error);
+  REQUIRE(rc == TILEDB_OK);
+  REQUIRE(error == nullptr);
+  rc = tiledb_config_set(
+      config, "sm.mem.consolidation.reader_weight", "5000", &error);
+  REQUIRE(rc == TILEDB_OK);
+  REQUIRE(error == nullptr);
+  rc = tiledb_config_set(
+      config, "sm.mem.consolidation.writer_weight", "5000", &error);
   REQUIRE(rc == TILEDB_OK);
   REQUIRE(error == nullptr);
 
@@ -1332,8 +1363,16 @@ TEST_CASE_METHOD(
   REQUIRE(tiledb_config_alloc(&config, &error) == TILEDB_OK);
   REQUIRE(error == nullptr);
 
-  int rc = tiledb_config_set(
-      config, "sm.consolidation.total_buffer_size", "1048576", &error);
+  auto rc = tiledb_config_set(
+      config, "sm.mem.consolidation.buffers_weight", "1", &error);
+  REQUIRE(rc == TILEDB_OK);
+  REQUIRE(error == nullptr);
+  rc = tiledb_config_set(
+      config, "sm.mem.consolidation.reader_weight", "5000", &error);
+  REQUIRE(rc == TILEDB_OK);
+  REQUIRE(error == nullptr);
+  rc = tiledb_config_set(
+      config, "sm.mem.consolidation.writer_weight", "5000", &error);
   REQUIRE(rc == TILEDB_OK);
   REQUIRE(error == nullptr);
 
