@@ -5,7 +5,7 @@
  *
  * The MIT License
  *
- * @copyright Copyright (c) 2023 TileDB Inc.
+ * @copyright Copyright (c) 2023-2024 TileDB Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -30,6 +30,7 @@
  * Tests the C++ API for schema evolution.
  */
 
+#include <test/support/src/vfs_helpers.h>
 #include <test/support/tdb_catch.h>
 #include "test/support/src/mem_helpers.h"
 #include "tiledb/sm/array_schema/array_schema.h"
@@ -45,14 +46,14 @@
 
 #include <limits>
 
+using namespace tiledb;
+
 TEST_CASE(
     "C++ API: SchemaEvolution, add and drop attributes",
-    "[cppapi][schema][evolution][add][drop]") {
-  using namespace tiledb;
-  Context ctx;
-  VFS vfs(ctx);
-
-  std::string array_uri = "test_schema_evolution_array";
+    "[cppapi][schema][evolution][add][drop][rest]") {
+  test::VFSTestSetup vfs_test_setup;
+  Context ctx{vfs_test_setup.ctx()};
+  auto array_uri{vfs_test_setup.array_uri("test_schema_evolution_array")};
 
   Domain domain(ctx);
   auto id1 = Dimension::create<int>(ctx, "d1", {{-100, 100}}, 10);
@@ -71,10 +72,6 @@ TEST_CASE(
   schema.set_cell_order(TILEDB_ROW_MAJOR);
   schema.set_tile_order(TILEDB_COL_MAJOR);
 
-  if (vfs.is_dir(array_uri)) {
-    vfs.remove_dir(array_uri);
-  }
-
   Array::create(array_uri, schema);
 
   auto evolution = ArraySchemaEvolution(ctx);
@@ -100,27 +97,54 @@ TEST_CASE(
   CHECK(attrs.count("a1") == 0);
   CHECK(attrs.count("a2") == 1);
   CHECK(attrs.count("a3") == 1);
+}
 
-  // Clean up
-  if (vfs.is_dir(array_uri)) {
-    vfs.remove_dir(array_uri);
-  }
+TEST_CASE(
+    "C++ API: SchemaEvolution, check error when dropping dimension",
+    "[cppapi][schema][evolution][drop][rest]") {
+  test::VFSTestSetup vfs_test_setup;
+  Context ctx{vfs_test_setup.ctx()};
+  auto array_uri{vfs_test_setup.array_uri("test_schema_evolution_array")};
+
+  Domain domain(ctx);
+  auto id1 = Dimension::create<int>(ctx, "d1", {{-100, 100}}, 10);
+  auto id2 = Dimension::create<int>(ctx, "d2", {{0, 100}}, 5);
+  domain.add_dimension(id1).add_dimension(id2);
+
+  auto a1 = Attribute::create<int>(ctx, "a1");
+  auto a2 = Attribute::create<int>(ctx, "a2");
+
+  ArraySchema schema(ctx, TILEDB_DENSE);
+  schema.set_domain(domain);
+  schema.add_attribute(a1);
+  schema.add_attribute(a2);
+  schema.set_cell_order(TILEDB_ROW_MAJOR);
+  schema.set_tile_order(TILEDB_COL_MAJOR);
+
+  Array::create(array_uri, schema);
+
+  auto evolution = ArraySchemaEvolution(ctx);
+
+  // try to drop dimension d1
+  evolution.drop_attribute("d1");
+
+  // check that an exception is thrown
+  CHECK_THROWS(evolution.array_evolve(array_uri));
 }
 
 TEST_CASE(
     "C++ API: SchemaEvolution, add attributes and read",
-    "[cppapi][schema][evolution][add]") {
-  using namespace tiledb;
-  Context ctx;
-  VFS vfs(ctx);
+    "[cppapi][schema][evolution][add][rest]") {
+  test::VFSTestSetup vfs_test_setup;
+  Context ctx{vfs_test_setup.ctx()};
+
   auto layout = GENERATE(
       TILEDB_ROW_MAJOR,
       TILEDB_COL_MAJOR,
       TILEDB_UNORDERED,
       TILEDB_GLOBAL_ORDER);
   bool duplicates = GENERATE(true, false);
-
-  std::string array_uri = "test_schema_evolution_array_read";
+  auto array_uri{vfs_test_setup.array_uri("test_schema_evolution_array")};
 
   // Create
   {
@@ -139,10 +163,6 @@ TEST_CASE(
     schema.set_cell_order(TILEDB_ROW_MAJOR);
     schema.set_tile_order(TILEDB_COL_MAJOR);
 
-    if (vfs.is_dir(array_uri)) {
-      vfs.remove_dir(array_uri);
-    }
-
     Array::create(array_uri, schema);
   }
 
@@ -180,8 +200,9 @@ TEST_CASE(
 
     // Prepare the query
     Query query(ctx, array, TILEDB_READ);
-    query.add_range(0, 1, 4)
-        .add_range(1, 1, 4)
+    Subarray subarray(ctx, array);
+    subarray.add_range(0, 1, 4).add_range(1, 1, 4);
+    query.set_subarray(subarray)
         .set_layout(layout)
         .set_data_buffer("a", data)
         .set_data_buffer("d1", d1_data)
@@ -301,8 +322,9 @@ TEST_CASE(
 
     // Prepare the query
     Query query(ctx, array, TILEDB_READ);
-    query.add_range(0, 1, 4)
-        .add_range(1, 1, 4)
+    Subarray subarray(ctx, array);
+    subarray.add_range(0, 1, 4).add_range(1, 1, 4);
+    query.set_subarray(subarray)
         .set_layout(layout)
         .set_data_buffer("a", a_data)
         .set_data_buffer("b", b_data)
@@ -418,9 +440,11 @@ TEST_CASE(
   // test case.
   Config cfg;
   cfg["sm.merge_overlapping_ranges_experimental"] = "false";
+  vfs_test_setup.update_config(cfg.ptr().get());
   // + Global order does not support multi-range subarrays
   if (layout != TILEDB_GLOBAL_ORDER) {
-    ctx = Context(cfg);
+    ctx = vfs_test_setup.ctx();
+
     Array array(ctx, array_uri, TILEDB_READ);
 
     std::vector<int> a_data(8);
@@ -452,10 +476,12 @@ TEST_CASE(
 
     // Prepare the query
     Query query(ctx, array, TILEDB_READ);
-    query.add_range(0, 1, 4)
-        .add_range(0, 1, 4)
-        .add_range(1, 1, 4)
-        .add_range(1, 1, 4)
+    Subarray subarray(ctx, array);
+    subarray.set_config(cfg);
+    subarray.add_range(0, 1, 4).add_range(0, 1, 4).add_range(1, 1, 4).add_range(
+        1, 1, 4);
+    subarray.set_config(cfg);
+    query.set_subarray(subarray)
         .set_layout(layout)
         .set_data_buffer("a", a_data)
         .set_data_buffer("b", b_data)
@@ -637,19 +663,13 @@ TEST_CASE(
               1, 1, 1, 1, 3, 3, 3, 3, 4, 4, 4, 4, 1, 1, 1, 1}));
     }
   }
-
-  // Clean up
-  if (vfs.is_dir(array_uri)) {
-    vfs.remove_dir(array_uri);
-  }
 }
 
 TEST_CASE(
     "C++ API: SchemaEvolution, add and drop attributes",
-    "[cppapi][schema][evolution][add][query-condition]") {
-  using namespace tiledb;
-  Context ctx;
-  VFS vfs(ctx);
+    "[cppapi][schema][evolution][add][query-condition][rest]") {
+  test::VFSTestSetup vfs_test_setup;
+  Context ctx{vfs_test_setup.ctx()};
   auto layout = GENERATE(
       TILEDB_ROW_MAJOR,
       TILEDB_COL_MAJOR,
@@ -659,7 +679,8 @@ TEST_CASE(
 
   const char* out_str = nullptr;
   tiledb_layout_to_str(layout, &out_str);
-  std::string array_uri = "test_schema_evolution_query_condition";
+  auto array_uri{
+      vfs_test_setup.array_uri("test_schema_evolution_query_condition")};
 
   {
     Domain domain(ctx);
@@ -677,10 +698,6 @@ TEST_CASE(
     schema.set_cell_order(TILEDB_ROW_MAJOR);
     schema.set_tile_order(TILEDB_COL_MAJOR);
 
-    if (vfs.is_dir(array_uri)) {
-      vfs.remove_dir(array_uri);
-    }
-
     Array::create(array_uri, schema);
   }
 
@@ -787,16 +804,106 @@ TEST_CASE(
     CHECK_THAT(d1_data, Catch::Matchers::Equals(std::vector<int>{4}));
     CHECK_THAT(d2_data, Catch::Matchers::Equals(std::vector<int>{1}));
   }
+}
 
-  // Cleanup.
-  if (vfs.is_dir(array_uri)) {
-    vfs.remove_dir(array_uri);
+TEST_CASE(
+    "C++ API: SchemaEvolution, drop fixed attribute and add back as var-sized",
+    "[cppapi][schema][evolution][add][drop]") {
+  test::VFSTestSetup vfs_test_setup;
+  Context ctx{vfs_test_setup.ctx()};
+  auto array_uri{
+      vfs_test_setup.array_uri("test_schema_evolution_drop_fixed_add_var")};
+  tiledb_array_type_t array_type = TILEDB_DENSE;
+  bool allows_dups = false;
+  auto layout = GENERATE(TILEDB_UNORDERED, TILEDB_GLOBAL_ORDER);
+
+  SECTION("sparse") {
+    array_type = TILEDB_SPARSE;
+    allows_dups = GENERATE(true, false);
   }
+
+  // Create array
+  Domain domain(ctx);
+  auto d = Dimension::create<int>(ctx, "d", {{1, 10}}, 1);
+  domain.add_dimension(d);
+  auto a = Attribute::create<int>(ctx, "a");
+  auto b = Attribute::create<int>(ctx, "b");
+  ArraySchema schema(ctx, array_type);
+  schema.set_domain(domain);
+  schema.set_allows_dups(allows_dups);
+  CHECK(allows_dups == schema.allows_dups());
+  schema.add_attribute(a);
+  schema.add_attribute(b);
+  schema.set_cell_order(TILEDB_ROW_MAJOR);
+  schema.set_tile_order(TILEDB_COL_MAJOR);
+  Array::create(array_uri, schema);
+
+  // Write a fragment to the array
+  std::vector<int> data = {1, 2, 3, 4, 5, 6, 7, 8, 9, 10};
+  Array array_w(ctx, array_uri, TILEDB_WRITE);
+  Query query_w(ctx, array_w, TILEDB_WRITE);
+  query_w.set_layout(TILEDB_GLOBAL_ORDER)
+      .set_data_buffer("a", data)
+      .set_data_buffer("b", data);
+  if (array_type == TILEDB_SPARSE) {
+    query_w.set_data_buffer("d", data);
+  }
+  query_w.submit_and_finalize();
+  array_w.close();
+  uint64_t fragment_write_ts = tiledb_timestamp_now_ms() + 1;
+  std::this_thread::sleep_for(std::chrono::milliseconds(100));
+
+  // Evolve schema to drop attribute "a"
+  ArraySchemaEvolution schema_evolution = ArraySchemaEvolution(ctx);
+  uint64_t now = tiledb_timestamp_now_ms() + 1;
+  schema_evolution.set_timestamp_range(std::make_pair(now, now));
+  schema_evolution.drop_attribute("a");
+  schema_evolution.array_evolve(array_uri);
+  std::this_thread::sleep_for(std::chrono::milliseconds(100));
+
+  // Evolve schema to add back attribute "a" as a string
+  auto a_new = Attribute::create<std::string>(ctx, "a");
+  now = tiledb_timestamp_now_ms() + 1;
+  schema_evolution.set_timestamp_range(std::make_pair(now, now));
+  schema_evolution.add_attribute(a_new);
+  schema_evolution.array_evolve(array_uri);
+
+  // Read the array
+  std::string buffer;
+  std::vector<uint64_t> offsets(10);
+  Array array_r(ctx, array_uri, TILEDB_READ);
+  Subarray subarray_r(ctx, array_r);
+  subarray_r.add_range(0, 1, 10);
+  Query query_r(ctx, array_r, TILEDB_READ);
+  query_r.set_layout(layout)
+      .set_subarray(subarray_r)
+      .set_data_buffer("a", buffer)
+      .set_offsets_buffer("a", offsets);
+  query_r.submit();
+  array_r.close();
+
+  // Read the original array
+  std::vector<int> a_data(10);
+  array_r.open(TILEDB_READ, fragment_write_ts);
+  Subarray subarray_r2(ctx, array_r);
+  subarray_r2.add_range(0, 1, 10);
+  Query query_r2(ctx, array_r, TILEDB_READ);
+  query_r2.set_layout(layout)
+      .set_subarray(subarray_r2)
+      .set_data_buffer("a", a_data);
+  query_r2.submit();
+  array_r.close();
+  auto result_num = (int)query_r2.result_buffer_elements()["a"].second;
+  CHECK(result_num == 10);
+  a_data.resize(result_num);
+  CHECK_THAT(
+      a_data,
+      Catch::Matchers::Equals(std::vector<int>{1, 2, 3, 4, 5, 6, 7, 8, 9, 10}));
 }
 
 TEST_CASE(
     "SchemaEvolution Error Handling Tests",
-    "[cppapi][schema][evolution][errors]") {
+    "[cppapi][schema][evolution][errors][rest]") {
   auto ase = make_shared<tiledb::sm::ArraySchemaEvolution>(
       HERE(), tiledb::test::create_test_memory_tracker());
   REQUIRE_THROWS(ase->evolve_schema(nullptr));
