@@ -5,7 +5,7 @@
  *
  * The MIT License
  *
- * @copyright Copyright (c) 2017-2023 TileDB Inc.
+ * @copyright Copyright (c) 2017-2024 TileDB Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -36,7 +36,6 @@
 #include "test/support/src/vfs_helpers.h"
 #include "tiledb/sm/c_api/tiledb_struct_def.h"
 #include "tiledb/sm/cpp_api/tiledb"
-#include "tiledb/sm/misc/utils.h"
 
 using namespace tiledb;
 using namespace tiledb::test;
@@ -80,8 +79,10 @@ TEST_CASE("C++ API: Test subarray", "[cppapi][sparse][subarray]") {
     tiledb::Array array(ctx, array_name, TILEDB_READ);
     tiledb::Query query(ctx, array);
     int range[] = {0, 0};
-    query.add_range(0, range[0], range[1]);
-    query.add_range(1, range[0], range[1]);
+    Subarray subarray(ctx, array);
+    subarray.add_range(0, range[0], range[1]);
+    subarray.add_range(1, range[0], range[1]);
+    query.set_subarray(subarray);
 
     auto est_size = query.est_result_size("a");
     REQUIRE(est_size == 4);
@@ -118,7 +119,9 @@ TEST_CASE("C++ API: Test subarray", "[cppapi][sparse][subarray]") {
     tiledb::Array array(ctx, array_name, TILEDB_READ);
     tiledb::Query query(ctx, array);
     int range[] = {1, 2};
-    query.add_range(0, range[0], range[1]).add_range(1, range[0], range[1]);
+    Subarray subarray(ctx, array);
+    subarray.add_range(0, range[0], range[1]).add_range(1, range[0], range[1]);
+    query.set_subarray(subarray);
 
     auto est_size = query.est_result_size("a");
     REQUIRE(est_size == 4);
@@ -158,27 +161,29 @@ TEST_CASE("C++ API: Test subarray", "[cppapi][sparse][subarray]") {
     tiledb::Array array(ctx, array_name, TILEDB_READ);
     tiledb::Query query(ctx, array);
     int range0[] = {0, 0}, range1[] = {2, 2};
-    query.add_range(0, range0[0], range0[1]);
-    query.add_range(1, range0[0], range0[1]);
-    query.add_range(0, range1[0], range1[1]);
-    query.add_range(1, range1[0], range1[1]);
+    Subarray subarray(ctx, array);
+    subarray.add_range(0, range0[0], range0[1]);
+    subarray.add_range(1, range0[0], range0[1]);
+    subarray.add_range(0, range1[0], range1[1]);
+    subarray.add_range(1, range1[0], range1[1]);
 
     int64_t inv_range[] = {0, 1};
-    CHECK_THROWS(query.add_range(1, inv_range[0], inv_range[1]));
+    CHECK_THROWS(subarray.add_range(1, inv_range[0], inv_range[1]));
+    query.set_subarray(subarray);
 
     // Get range
-    auto range = query.range<int>(0, 0);
+    auto range = subarray.range<int>(0, 0);
     CHECK(range[0] == 0);
     CHECK(range[1] == 0);
     CHECK(range[2] == 0);
-    range = query.range<int>(1, 1);
+    range = subarray.range<int>(1, 1);
     CHECK(range[0] == 2);
     CHECK(range[1] == 2);
     CHECK(range[2] == 0);
 
-    CHECK_THROWS(range = query.range<int>(1, 3));
+    CHECK_THROWS(range = subarray.range<int>(1, 3));
     std::array<int64_t, 3> range2 = {{0, 0, 0}};
-    CHECK_THROWS(range2 = query.range<int64_t>(1, 1));
+    CHECK_THROWS(range2 = subarray.range<int64_t>(1, 1));
 
     auto est_size = query.est_result_size("a");
     REQUIRE(est_size == 4);
@@ -235,10 +240,12 @@ TEST_CASE("C++ API: Test subarray", "[cppapi][sparse][subarray]") {
     tiledb::Array array(ctx, array_name, TILEDB_READ);
     tiledb::Query query(ctx, array);
     int range0[] = {0, 1}, range1[] = {2, 3};
-    query.add_range(0, range0[0], range0[1]);
-    query.add_range(1, range0[0], range0[1]);
-    query.add_range(0, range1[0], range1[1]);
-    query.add_range(1, range1[0], range1[1]);
+    Subarray subarray(ctx, array);
+    subarray.add_range(0, range0[0], range0[1]);
+    subarray.add_range(1, range0[0], range0[1]);
+    subarray.add_range(0, range1[0], range1[1]);
+    subarray.add_range(1, range1[0], range1[1]);
+    query.set_subarray(subarray);
 
     auto est_size = query.est_result_size("a");
     std::vector<int> data(est_size);
@@ -282,8 +289,10 @@ TEST_CASE("C++ API: Test subarray", "[cppapi][sparse][subarray]") {
     query.set_config(config);
     int range[] = {1, 4};
     int range2[] = {-1, 3};
-    REQUIRE_THROWS(query.add_range(0, range[0], range[1]));
-    REQUIRE_THROWS(query.add_range(1, range2[0], range2[1]));
+    Subarray subarray(ctx, array);
+    REQUIRE_THROWS(subarray.add_range(0, range[0], range[1]));
+    REQUIRE_THROWS(subarray.add_range(1, range2[0], range2[1]));
+    query.set_subarray(subarray);
   }
 
   SECTION("- Read ranges oob error - Subarray-cppapi") {
@@ -319,8 +328,11 @@ TEST_CASE("C++ API: Test subarray", "[cppapi][sparse][subarray]") {
     query.set_config(config);
     int range[] = {1, 4};
     int range2[] = {-1, 3};
-    query.add_range(0, range[0], range[1]);
-    query.add_range(1, range2[0], range2[1]);
+    Subarray subarray(ctx, array);
+    subarray.set_config(config);
+    subarray.add_range(0, range[0], range[1]);
+    subarray.add_range(1, range2[0], range2[1]);
+    query.set_subarray(subarray);
 
     auto est_size = query.est_result_size("a");
     REQUIRE(est_size == 12);
@@ -335,18 +347,6 @@ TEST_CASE("C++ API: Test subarray", "[cppapi][sparse][subarray]") {
     REQUIRE(data[1] == 3);
   }
 
-  SECTION("- set_subarray  Write ranges oob warn - Subarray-query") {
-    tiledb::Array array(ctx, array_name, TILEDB_WRITE);
-    tiledb::Query query(ctx, array);
-    // throws on sparse array (set_subarray not supported)
-    REQUIRE_THROWS(query.set_subarray({1, 4, -1, 3}));
-    tiledb::Config config;
-    config.set("sm.read_range_oob", "warn");
-    query.set_config(config);
-    // throws on sparse array
-    REQUIRE_THROWS(query.set_subarray({1, 4, -1, 3}));
-  }
-
   SECTION("- Read ranges oob warn - Subarray-cppapi") {
     tiledb::Array array(ctx, array_name, TILEDB_READ);
     tiledb::Subarray subarray(ctx, array);
@@ -468,29 +468,29 @@ TEST_CASE(
   SECTION("- Upper bound OOB") {
     int range[] = {0, 100};
     auto r = Range(&range[0], &range[1], sizeof(int));
-    CHECK_NOTHROW(subarray.ptr().get()->subarray_->add_range_unsafe(0, r));
+    CHECK_NOTHROW(subarray.ptr().get()->add_range_unsafe(0, r));
   }
 
   SECTION("- Lower bound OOB") {
     int range[] = {-1, 2};
     auto r = Range(&range[0], &range[1], sizeof(int));
-    CHECK_NOTHROW(subarray.ptr().get()->subarray_->add_range_unsafe(0, r));
+    CHECK_NOTHROW(subarray.ptr().get()->add_range_unsafe(0, r));
   }
 
   SECTION("- Second range OOB") {
     int range[] = {1, 4};
     auto r = Range(&range[0], &range[1], sizeof(int));
-    CHECK_NOTHROW(subarray.ptr().get()->subarray_->add_range_unsafe(0, r));
+    CHECK_NOTHROW(subarray.ptr().get()->add_range_unsafe(0, r));
     int range2[] = {10, 20};
     auto r2 = Range(&range2[0], &range2[1], sizeof(int));
-    CHECK_NOTHROW(subarray.ptr().get()->subarray_->add_range_unsafe(1, r2));
+    CHECK_NOTHROW(subarray.ptr().get()->add_range_unsafe(1, r2));
   }
 
   SECTION("- Valid ranges") {
     int range[] = {0, 1};
     auto r = Range(&range[0], &range[1], sizeof(int));
-    CHECK_NOTHROW(subarray.ptr().get()->subarray_->add_range_unsafe(0, r));
-    CHECK_NOTHROW(subarray.ptr().get()->subarray_->add_range_unsafe(1, r));
+    CHECK_NOTHROW(subarray.ptr().get()->add_range_unsafe(0, r));
+    CHECK_NOTHROW(subarray.ptr().get()->add_range_unsafe(1, r));
     expected = TILEDB_OK;
   }
 
@@ -572,14 +572,16 @@ TEST_CASE(
   int row_range[] = {0, 1};
   int col_range0[] = {12277, 13499};
   int col_range1[] = {13500, 17486};
-  query.add_range(0, row_range[0], row_range[1]);
-  query.add_range(1, col_range0[0], col_range0[1]);
-  query.add_range(1, col_range1[0], col_range1[1]);
+  Subarray subarray(ctx, array);
+  subarray.add_range(0, row_range[0], row_range[1]);
+  subarray.add_range(1, col_range0[0], col_range0[1]);
+  subarray.add_range(1, col_range1[0], col_range1[1]);
+  query.set_subarray(subarray);
 
   // Test range num
-  auto range_num = query.range_num(0);
+  auto range_num = subarray.range_num(0);
   CHECK(range_num == 1);
-  range_num = query.range_num(1);
+  range_num = subarray.range_num(1);
   // Ranges `col_range0` and `col_range1` are coalesced.
   CHECK(range_num == 1);
 
@@ -820,14 +822,14 @@ TEST_CASE(
     // since the other CHECKS(range_num == 1) succeed whether or not the
     //.set_subarray() was done (accidental discovery.)
     CHECK(!subarray_equiv<int>(
-        *default_subarray_from_query.ptr().get()->subarray_,
-        *subarray.ptr().get()->subarray_));
+        *default_subarray_from_query.ptr()->subarray().get(),
+        *subarray.ptr()->subarray().get()));
     query.set_subarray(subarray);
     tiledb::Subarray retrieved_query_subarray(query.ctx(), query.array());
     query.update_subarray_from_query(&retrieved_query_subarray);
     CHECK(subarray_equiv<int>(
-        *retrieved_query_subarray.ptr().get()->subarray_,
-        *subarray.ptr().get()->subarray_));
+        *retrieved_query_subarray.ptr()->subarray().get(),
+        *subarray.ptr()->subarray().get()));
     // Test range num
     auto range_num = retrieved_query_subarray.range_num(0);
     CHECK(range_num == 1);
@@ -1034,9 +1036,12 @@ TEST_CASE(
   int row_range[] = {1, 1};
   int col_range0[] = {12277, 12277};
   int col_range1[] = {12277, 13160};
-  query.add_range(0, row_range[0], row_range[1]);
-  query.add_range(1, col_range0[0], col_range0[1]);
-  query.add_range(1, col_range1[0], col_range1[1]);
+  Subarray subarray(ctx, array);
+  subarray.add_range(0, row_range[0], row_range[1]);
+  subarray.add_range(1, col_range0[0], col_range0[1]);
+  subarray.add_range(1, col_range1[0], col_range1[1]);
+  subarray.set_config(cfg);
+  query.set_subarray(subarray);
   query.set_layout(TILEDB_UNORDERED);
 
   // Allocate buffers large enough to hold 2 cells at a time.
@@ -1169,6 +1174,7 @@ TEST_CASE(
       .set_data_buffer("a", data);
 
   // Submit query
+  subarray.set_config(cfg);
   query.set_subarray(subarray);
   auto st = query.submit();
   REQUIRE(st == tiledb::Query::Status::INCOMPLETE);
@@ -1287,7 +1293,7 @@ TEST_CASE(
     tiledb_subarray_serialize(ctx.ptr().get(), array.ptr().get(), &ptr);
     subarray.replace_subarray_data(ptr);
   }
-  CHECK(coalesce == subarray.ptr()->subarray_->coalesce_ranges());
+  CHECK(coalesce == subarray.ptr()->coalesce_ranges());
 
   subarray.add_range(0, 1, 10);
   subarray.add_range(0, 11, 20);
