@@ -5,7 +5,7 @@
  *
  * The MIT License
  *
- * @copyright Copyright (c) 2023 TileDB Inc.
+ * @copyright Copyright (c) 2023-2024 TileDB Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -30,6 +30,7 @@
  * Tests the C++ API for enumeration related functions.
  */
 
+#include <numeric>
 #include <sstream>
 
 #include "test/support/src/mem_helpers.h"
@@ -137,8 +138,6 @@ struct EnumerationFx {
   template <typename T>
   bool vec_cmp(std::vector<T> v1, std::vector<T> v2);
 
-  void flatten_buffer_list(BufferList& blist, Buffer& buf);
-
   void rm_array();
 
   shared_ptr<MemoryTracker> memory_tracker_;
@@ -1093,7 +1092,7 @@ TEST_CASE_METHOD(
     EnumerationFx,
     "Array - Get Enumeration Error - Not Open",
     "[enumeration][array][error][not-open]") {
-  auto array = make_shared<Array>(HERE(), uri_, ctx_.storage_manager());
+  auto array = make_shared<Array>(HERE(), ctx_.resources(), uri_);
   auto matcher = Catch::Matchers::ContainsSubstring("Array is not open");
   REQUIRE_THROWS(array->get_enumeration("foo"), matcher);
 }
@@ -1124,15 +1123,16 @@ TEST_CASE_METHOD(
     "Array - Load All Enumerations - Repeated",
     "[enumeration][array][load-all-enumerations][repeated]") {
   create_array();
+  bool all_schemas = GENERATE(true, false);
   auto array = get_array(QueryType::READ);
   auto schema = array->array_schema_latest_ptr();
 
   REQUIRE(schema->is_enumeration_loaded("test_enmr") == false);
 
-  CHECK_NOTHROW(array->load_all_enumerations());
+  CHECK_NOTHROW(array->load_all_enumerations(all_schemas));
   REQUIRE(schema->is_enumeration_loaded("test_enmr") == true);
 
-  CHECK_NOTHROW(array->load_all_enumerations());
+  CHECK_NOTHROW(array->load_all_enumerations(all_schemas));
   REQUIRE(schema->is_enumeration_loaded("test_enmr") == true);
 }
 
@@ -1140,9 +1140,10 @@ TEST_CASE_METHOD(
     EnumerationFx,
     "Array - Load All Enumerations Error - Not Open",
     "[enumeration][array][error][not-open]") {
-  auto array = make_shared<Array>(HERE(), uri_, ctx_.storage_manager());
+  bool all_schemas = GENERATE(true, false);
+  auto array = make_shared<Array>(HERE(), ctx_.resources(), uri_);
   auto matcher = Catch::Matchers::ContainsSubstring("Array is not open");
-  REQUIRE_THROWS(array->load_all_enumerations());
+  REQUIRE_THROWS(array->load_all_enumerations(all_schemas));
 }
 
 /* ********************************* */
@@ -1632,8 +1633,9 @@ TEST_CASE_METHOD(
     "ArraySchemaEvolution - Simple No Enumeration",
     "[enumeration][array-schema-evolution][simple]") {
   create_array();
+  bool all_schemas = GENERATE(true, false);
   auto array = get_array(QueryType::READ);
-  array->load_all_enumerations();
+  array->load_all_enumerations(all_schemas);
 
   auto orig_schema = array->array_schema_latest_ptr();
   auto ase = make_shared<ArraySchemaEvolution>(HERE(), memory_tracker_);
@@ -2064,9 +2066,11 @@ TEST_CASE_METHOD(
 
   auto ase = make_shared<ArraySchemaEvolution>(HERE(), memory_tracker_);
   ase->extend_enumeration(new_enmr);
-  auto st = ctx_.storage_manager()->array_evolve_schema(
-      array->array_uri(), ase.get(), array->get_encryption_key());
-  throw_if_not_ok(st);
+  tiledb::sm::Array::evolve_array_schema(
+      ctx_.resources(),
+      array->array_uri(),
+      ase.get(),
+      array->get_encryption_key());
 
   // Check that we can not rewrite the query condition.
   array = get_array(QueryType::READ);
@@ -2396,10 +2400,18 @@ TEST_CASE_METHOD(
   auto qc1 = create_qc("attr1", (int)2, QueryConditionOp::EQ);
   qc1.set_use_enumeration(false);
 
-  Query q1(ctx_.storage_manager(), array);
+  Query q1(
+      ctx_.resources(),
+      ctx_.cancellation_source(),
+      ctx_.storage_manager(),
+      array);
   throw_if_not_ok(q1.set_condition(qc1));
 
-  Query q2(ctx_.storage_manager(), array);
+  Query q2(
+      ctx_.resources(),
+      ctx_.cancellation_source(),
+      ctx_.storage_manager(),
+      array);
   ser_des_query(&q1, &q2, client_side, ser_type);
 
   auto qc2 = q2.condition();
@@ -2432,10 +2444,18 @@ TEST_CASE_METHOD(
 
   throw_if_not_ok(qc1.combine(qc2, QueryConditionCombinationOp::OR, &qc3));
 
-  Query q1(ctx_.storage_manager(), array);
+  Query q1(
+      ctx_.resources(),
+      ctx_.cancellation_source(),
+      ctx_.storage_manager(),
+      array);
   throw_if_not_ok(q1.set_condition(qc3));
 
-  Query q2(ctx_.storage_manager(), array);
+  Query q2(
+      ctx_.resources(),
+      ctx_.cancellation_source(),
+      ctx_.storage_manager(),
+      array);
   ser_des_query(&q1, &q2, client_side, ser_type);
 
   auto qc4 = q2.condition();
@@ -2462,7 +2482,7 @@ TEST_CASE_METHOD(
   throw_if_not_ok(cfg.set("rest.load_enumerations_on_array_open", do_load));
   Context ctx(cfg);
 
-  auto a1 = make_shared<Array>(HERE(), uri_, ctx.storage_manager());
+  auto a1 = make_shared<Array>(HERE(), ctx.resources(), uri_);
   throw_if_not_ok(
       a1->open(QueryType::READ, EncryptionType::NO_ENCRYPTION, nullptr, 0));
   REQUIRE(a1->serialize_enumerations() == (do_load == "true"));
@@ -2470,7 +2490,7 @@ TEST_CASE_METHOD(
       a1->array_schema_latest_ptr()->get_loaded_enumeration_names().size() ==
       0);
 
-  auto a2 = make_shared<Array>(HERE(), uri_, ctx.storage_manager());
+  auto a2 = make_shared<Array>(HERE(), ctx.resources(), uri_);
 
   ser_des_array(ctx, a1.get(), a2.get(), client_side, ser_type);
 
@@ -2879,11 +2899,11 @@ shared_ptr<ArraySchema> EnumerationFx::create_schema() {
 
 void EnumerationFx::create_array() {
   auto schema = create_schema();
-  throw_if_not_ok(ctx_.storage_manager()->array_create(uri_, schema, enc_key_));
+  Array::create(ctx_.resources(), uri_, schema, enc_key_);
 }
 
 shared_ptr<Array> EnumerationFx::get_array(QueryType type) {
-  auto array = make_shared<Array>(HERE(), uri_, ctx_.storage_manager());
+  auto array = make_shared<Array>(HERE(), ctx_.resources(), uri_);
   throw_if_not_ok(array->open(type, EncryptionType::NO_ENCRYPTION, nullptr, 0));
   return array;
 }
@@ -2904,34 +2924,41 @@ shared_ptr<ArraySchema> EnumerationFx::ser_des_array_schema(
     shared_ptr<const ArraySchema> schema,
     bool client_side,
     SerializationType stype) {
-  Buffer buf;
+  SerializationBuffer buf{tiledb::test::get_test_memory_tracker()->get_resource(
+      MemoryType::SERIALIZATION_BUFFER)};
   throw_if_not_ok(serialization::array_schema_serialize(
-      *(schema.get()), stype, &buf, client_side));
+      *(schema.get()), stype, buf, client_side));
   return serialization::array_schema_deserialize(stype, buf, memory_tracker_);
 }
 
 shared_ptr<ArraySchemaEvolution> EnumerationFx::ser_des_array_schema_evolution(
     ArraySchemaEvolution* ase, bool client_side, SerializationType stype) {
-  Buffer buf;
+  SerializationBuffer buf{tiledb::test::get_test_memory_tracker()->get_resource(
+      MemoryType::SERIALIZATION_BUFFER)};
   throw_if_not_ok(serialization::array_schema_evolution_serialize(
-      ase, stype, &buf, client_side));
+      ase, stype, buf, client_side));
 
   ArraySchemaEvolution* ret;
   throw_if_not_ok(serialization::array_schema_evolution_deserialize(
-      &ret, stype, buf, memory_tracker_));
+      &ret, cfg_, stype, buf, memory_tracker_));
 
   return shared_ptr<ArraySchemaEvolution>(ret);
 }
 
 void EnumerationFx::ser_des_query(
     Query* q_in, Query* q_out, bool client_side, SerializationType stype) {
-  Buffer buf;
-  BufferList blist;
+  auto tracker = tiledb::test::get_test_memory_tracker();
+  BufferList blist{tracker};
 
   throw_if_not_ok(
-      serialization::query_serialize(q_in, stype, client_side, &blist));
+      serialization::query_serialize(q_in, stype, client_side, blist));
 
-  flatten_buffer_list(blist, buf);
+  const auto nbytes = blist.total_size();
+  SerializationBuffer buf{
+      nbytes, tracker->get_resource(MemoryType::SERIALIZATION_BUFFER)};
+
+  blist.reset_offset();
+  blist.read(buf.owned_mutable_span().data(), nbytes);
 
   throw_if_not_ok(serialization::query_deserialize(
       buf,
@@ -2939,7 +2966,8 @@ void EnumerationFx::ser_des_query(
       client_side,
       nullptr,
       q_out,
-      &(ctx_.resources().compute_tp())));
+      &(ctx_.resources().compute_tp()),
+      tracker));
 }
 
 void EnumerationFx::ser_des_array(
@@ -2948,10 +2976,11 @@ void EnumerationFx::ser_des_array(
     Array* out,
     bool client_side,
     SerializationType stype) {
-  Buffer buf;
-  throw_if_not_ok(serialization::array_serialize(in, stype, &buf, client_side));
-  throw_if_not_ok(serialization::array_deserialize(
-      out, stype, buf, ctx.storage_manager(), memory_tracker_));
+  SerializationBuffer buf{tiledb::test::get_test_memory_tracker()->get_resource(
+      MemoryType::SERIALIZATION_BUFFER)};
+  throw_if_not_ok(serialization::array_serialize(in, stype, buf, client_side));
+  serialization::array_deserialize(
+      out, stype, buf, ctx.resources(), memory_tracker_);
 }
 
 #else  // No TILEDB_SERIALIZATION
@@ -2995,15 +3024,6 @@ bool EnumerationFx::vec_cmp(std::vector<T> v1, std::vector<T> v2) {
   return true;
 }
 
-void EnumerationFx::flatten_buffer_list(BufferList& blist, Buffer& buf) {
-  const auto nbytes = blist.total_size();
-  throw_if_not_ok(buf.realloc(nbytes));
-
-  blist.reset_offset();
-  throw_if_not_ok(blist.read(buf.data(), nbytes));
-  buf.set_size(nbytes);
-}
-
 void EnumerationFx::rm_array() {
   bool is_dir;
   throw_if_not_ok(ctx_.resources().vfs().is_dir(uri_, &is_dir));
