@@ -5,7 +5,7 @@
  *
  * The MIT License
  *
- * @copyright Copyright (c) 2017-2023 TileDB, Inc.
+ * @copyright Copyright (c) 2017-2024 TileDB, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -41,6 +41,7 @@
 #include <test/support/tdb_catch.h>
 #include "helpers.h"
 #include "serialization_wrappers.h"
+#include "tiledb/api/c_api/array/array_api_internal.h"
 #include "tiledb/api/c_api/buffer/buffer_api_internal.h"
 #include "tiledb/api/c_api/context/context_api_external.h"
 #include "tiledb/api/c_api/context/context_api_internal.h"
@@ -56,6 +57,7 @@
 #include "tiledb/sm/misc/tile_overlap.h"
 #include "tiledb/sm/serialization/array.h"
 #include "tiledb/sm/serialization/query.h"
+#include "tiledb/storage_format/uri/generate_uri.h"
 
 int setenv_local(const char* __name, const char* __value) {
 #ifdef _WIN32
@@ -822,8 +824,7 @@ void create_subarray(
     bool coalesce_ranges) {
   (void)layout;
   int32_t rc;
-  tiledb_array_t tdb_array;
-  tdb_array.array_ = array;
+  tiledb_array_t tdb_array = *tiledb_array_t::make_handle(array);
   rc = tiledb_subarray_alloc(ctx, &tdb_array, subarray);
   REQUIRE(rc == TILEDB_OK);
   if (rc == TILEDB_OK) {
@@ -1180,16 +1181,24 @@ void read_array(
   rc = tiledb_query_set_layout(ctx, query, layout);
   CHECK(rc == TILEDB_OK);
 
+  // Create subarray
+  tiledb_subarray_t* subarray;
+  rc = tiledb_subarray_alloc(ctx, array, &subarray);
+  CHECK(rc == TILEDB_OK);
+
   auto dim_num = (unsigned)ranges.size();
   for (unsigned i = 0; i < dim_num; ++i) {
     auto dim_range_num = ranges[i].size() / 2;
     for (size_t j = 0; j < dim_range_num; ++j) {
-      rc = tiledb_query_add_range(
-          ctx, query, i, &ranges[i][2 * j], &ranges[i][2 * j + 1], nullptr);
+      rc = tiledb_subarray_add_range(
+          ctx, subarray, i, &ranges[i][2 * j], &ranges[i][2 * j + 1], nullptr);
       CHECK(rc == TILEDB_OK);
     }
   }
 
+  // Set the subarray
+  tiledb_query_set_subarray_t(ctx, query, subarray);
+
   // Set buffers
   for (const auto& b : buffers) {
     if (b.second.var_ == nullptr) {  // Fixed-sized
@@ -1230,6 +1239,7 @@ void read_array(
 
   // Clean up
   tiledb_query_free(&query);
+  tiledb_subarray_free(&subarray);
 }
 
 int32_t num_commits(Context ctx, const std::string& array_name) {
@@ -1447,6 +1457,190 @@ int deserialize_array_and_query(
       ctx, c_buff, TILEDB_CAPNP, clientside ? 1 : 0, array_uri, query, &array);
 }
 
+sm::URI generate_fragment_uri(sm::Array* array) {
+  if (array == nullptr) {
+    return sm::URI(tiledb::storage_format::generate_timestamped_name(
+        TILEDB_TIMESTAMP_NOW_MS, constants::format_version));
+  }
+
+  uint64_t timestamp = array->timestamp_end_opened_at();
+  auto write_version = array->array_schema_latest().write_version();
+
+  auto new_fragment_str = tiledb::storage_format::generate_timestamped_name(
+      timestamp, write_version);
+  auto frag_dir_uri = array->array_directory().get_fragments_dir(write_version);
+  return frag_dir_uri.join_path(new_fragment_str);
+}
+
+void create_sparse_array_v11(tiledb_ctx_t* ctx, const std::string& array_name) {
+  tiledb_config_t* config;
+  REQUIRE(tiledb_ctx_get_config(ctx, &config) == TILEDB_OK);
+  tiledb_vfs_t* vfs;
+  REQUIRE(tiledb_vfs_alloc(ctx, config, &vfs) == TILEDB_OK);
+  // Get the v11 sparse array.
+  std::string v11_arrays_dir =
+      std::string(TILEDB_TEST_INPUTS_DIR) + "/arrays/sparse_array_v11";
+  REQUIRE(
+      tiledb_vfs_copy_dir(
+          ctx, vfs, v11_arrays_dir.c_str(), array_name.c_str()) == TILEDB_OK);
+}
+
+void write_sparse_v11(
+    tiledb_ctx_t* ctx, const std::string& array_name, uint64_t timestamp) {
+  // Prepare cell buffers.
+  std::vector<int> buffer_a1{0, 1, 2, 3};
+  std::vector<uint64_t> buffer_a2{0, 1, 3, 6};
+  std::string buffer_var_a2("abbcccdddd");
+  std::vector<float> buffer_a3{0.1f, 0.2f, 1.1f, 1.2f, 2.1f, 2.2f, 3.1f, 3.2f};
+  std::vector<uint64_t> buffer_coords_dim1{1, 1, 1, 2};
+  std::vector<uint64_t> buffer_coords_dim2{1, 2, 4, 3};
+
+  // Open array.
+  tiledb_array_t* array;
+  REQUIRE(tiledb_array_alloc(ctx, array_name.c_str(), &array) == TILEDB_OK);
+  REQUIRE(
+      tiledb_array_set_open_timestamp_end(ctx, array, timestamp) == TILEDB_OK);
+  REQUIRE(tiledb_array_open(ctx, array, TILEDB_WRITE) == TILEDB_OK);
+
+  // Create query.
+  tiledb_query_t* query;
+  REQUIRE(tiledb_query_alloc(ctx, array, TILEDB_WRITE, &query) == TILEDB_OK);
+  REQUIRE(
+      tiledb_query_set_layout(ctx, query, TILEDB_GLOBAL_ORDER) == TILEDB_OK);
+  uint64_t a1_size = buffer_a1.size() * sizeof(int);
+  REQUIRE(
+      tiledb_query_set_data_buffer(
+          ctx, query, "a1", buffer_a1.data(), &a1_size) == TILEDB_OK);
+  uint64_t a2_var_size = buffer_var_a2.size() * sizeof(char);
+  REQUIRE(
+      tiledb_query_set_data_buffer(
+          ctx, query, "a2", (void*)buffer_var_a2.c_str(), &a2_var_size) ==
+      TILEDB_OK);
+  uint64_t a2_size = buffer_a2.size() * sizeof(uint64_t);
+  REQUIRE(
+      tiledb_query_set_offsets_buffer(
+          ctx, query, "a2", buffer_a2.data(), &a2_size) == TILEDB_OK);
+  uint64_t a3_size = buffer_a3.size() * sizeof(float);
+  REQUIRE(
+      tiledb_query_set_data_buffer(
+          ctx, query, "a3", buffer_a3.data(), &a3_size) == TILEDB_OK);
+
+  uint64_t d1_size = buffer_coords_dim1.size() * sizeof(uint64_t);
+  REQUIRE(
+      tiledb_query_set_data_buffer(
+          ctx, query, "d1", buffer_coords_dim1.data(), &d1_size) == TILEDB_OK);
+  uint64_t d2_size = buffer_coords_dim2.size() * sizeof(uint64_t);
+  REQUIRE(
+      tiledb_query_set_data_buffer(
+          ctx, query, "d2", buffer_coords_dim2.data(), &d2_size) == TILEDB_OK);
+
+  // Submit/finalize the query.
+  REQUIRE(tiledb_query_submit_and_finalize(ctx, query) == TILEDB_OK);
+  // Close array.
+  REQUIRE(tiledb_array_close(ctx, array) == TILEDB_OK);
+
+  tiledb_array_free(&array);
+  tiledb_query_free(&query);
+}
+
+void read_sparse_v11(
+    tiledb_ctx_t* ctx, const std::string& array_name, uint64_t timestamp) {
+  // Prepare expected results for cell buffers.
+  std::vector<int> buffer_a1{0, 1, 2, 3};
+  std::vector<uint64_t> buffer_a2{0, 1, 3, 6};
+  std::string buffer_var_a2("abbcccdddd");
+  std::vector<float> buffer_a3{0.1f, 0.2f, 1.1f, 1.2f, 2.1f, 2.2f, 3.1f, 3.2f};
+  std::vector<uint64_t> buffer_coords_dim1{1, 1, 1, 2};
+  std::vector<uint64_t> buffer_coords_dim2{1, 2, 4, 3};
+
+  int buffer_a1_read[4];
+  uint64_t buffer_a2_read[4];
+  char buffer_var_a2_read[10];
+  float buffer_a3_read[8];
+  uint64_t buffer_coords_dim1_read[4];
+  uint64_t buffer_coords_dim2_read[4];
+
+  // Open array.
+  tiledb_array_t* array;
+  REQUIRE(tiledb_array_alloc(ctx, array_name.c_str(), &array) == TILEDB_OK);
+  REQUIRE(
+      tiledb_array_set_open_timestamp_end(ctx, array, timestamp) == TILEDB_OK);
+  REQUIRE(tiledb_array_open(ctx, array, TILEDB_READ) == TILEDB_OK);
+
+  // Create query.
+  tiledb_query_t* query;
+  REQUIRE(tiledb_query_alloc(ctx, array, TILEDB_READ, &query) == TILEDB_OK);
+  REQUIRE(
+      tiledb_query_set_layout(ctx, query, TILEDB_GLOBAL_ORDER) == TILEDB_OK);
+  uint64_t a1_size = buffer_a1.size() * sizeof(int);
+  REQUIRE(
+      tiledb_query_set_data_buffer(
+          ctx, query, "a1", buffer_a1_read, &a1_size) == TILEDB_OK);
+  uint64_t a2_var_size = buffer_var_a2.size() * sizeof(char);
+  REQUIRE(
+      tiledb_query_set_data_buffer(
+          ctx, query, "a2", buffer_var_a2_read, &a2_var_size) == TILEDB_OK);
+  uint64_t a2_size = buffer_a2.size() * sizeof(uint64_t);
+  REQUIRE(
+      tiledb_query_set_offsets_buffer(
+          ctx, query, "a2", buffer_a2_read, &a2_size) == TILEDB_OK);
+  uint64_t a3_size = buffer_a3.size() * sizeof(float);
+  REQUIRE(
+      tiledb_query_set_data_buffer(
+          ctx, query, "a3", buffer_a3_read, &a3_size) == TILEDB_OK);
+
+  uint64_t d1_size = buffer_coords_dim1.size() * sizeof(uint64_t);
+  REQUIRE(
+      tiledb_query_set_data_buffer(
+          ctx, query, "d1", buffer_coords_dim1_read, &d1_size) == TILEDB_OK);
+  uint64_t d2_size = buffer_coords_dim2.size() * sizeof(uint64_t);
+  REQUIRE(
+      tiledb_query_set_data_buffer(
+          ctx, query, "d2", buffer_coords_dim2_read, &d2_size) == TILEDB_OK);
+
+  // Submit the query.
+  REQUIRE(tiledb_query_submit(ctx, query) == TILEDB_OK);
+  // Close array.
+  REQUIRE(tiledb_array_close(ctx, array) == TILEDB_OK);
+
+  CHECK(!memcmp(buffer_a1.data(), buffer_a1_read, sizeof(buffer_a1_read)));
+  CHECK(!memcmp(
+      buffer_var_a2.data(), buffer_var_a2_read, sizeof(buffer_var_a2_read)));
+  CHECK(!memcmp(buffer_a2.data(), buffer_a2_read, sizeof(buffer_a2_read)));
+  CHECK(!memcmp(buffer_a3.data(), buffer_a3_read, sizeof(buffer_a3_read)));
+  CHECK(!memcmp(
+      buffer_coords_dim1.data(),
+      buffer_coords_dim1_read,
+      sizeof(buffer_coords_dim1_read)));
+  CHECK(!memcmp(
+      buffer_coords_dim2.data(),
+      buffer_coords_dim2_read,
+      sizeof(buffer_coords_dim2_read)));
+
+  tiledb_array_free(&array);
+  tiledb_query_free(&query);
+}
+
+void schema_equiv(
+    const sm::ArraySchema& schema1, const sm::ArraySchema& schema2) {
+  CHECK(schema1.array_type() == schema2.array_type());
+  CHECK(schema1.attributes().size() == schema2.attributes().size());
+  for (unsigned int i = 0; i < schema2.attribute_num(); i++) {
+    auto a = schema1.attribute(i);
+    auto b = schema2.attribute(i);
+    CHECK(a->cell_val_num() == b->cell_val_num());
+    CHECK(a->name() == b->name());
+    CHECK(a->type() == b->type());
+    CHECK(a->nullable() == b->nullable());
+    CHECK(a->get_enumeration_name() == b->get_enumeration_name());
+  }
+  CHECK(schema1.capacity() == schema2.capacity());
+  CHECK(schema1.cell_order() == schema2.cell_order());
+  CHECK(schema1.tile_order() == schema2.tile_order());
+  CHECK(schema1.allows_dups() == schema2.allows_dups());
+  CHECK(schema1.array_uri().to_string() == schema2.array_uri().to_string());
+}
+
 template void check_subarray<int8_t>(
     tiledb::sm::Subarray& subarray, const SubarrayRanges<int8_t>& ranges);
 template void check_subarray<uint8_t>(
