@@ -123,7 +123,7 @@ shared_ptr<ArraySchema> ArrayDirectory::load_array_schema_latest(
   auto&& array_schema = load_array_schema_from_uri(
       resources_.get(), schema_uri, encryption_key, memory_tracker);
 
-  array_schema->set_array_uri(uri_);
+  array_schema->set_array_uri(uri_.remove_trailing_slash());
 
   return std::move(array_schema);
 }
@@ -177,7 +177,10 @@ ArrayDirectory::load_all_array_schemas(
           array_schema->set_array_uri(uri_);
           schema_vector[schema_ith] = array_schema;
         } catch (std::exception& e) {
-          return Status_ArrayDirectoryError(e.what());
+          // TODO: We could throw a nested exception, but converting exceptions
+          // to statuses loses the inner exception messages. We can revisit this
+          // when Status gets removed from this module.
+          throw ArrayDirectoryException(e.what());
         }
 
         return Status::Ok();
@@ -316,13 +319,12 @@ void ArrayDirectory::delete_fragments_list(
   // Delete fragments and commits
   throw_if_not_ok(parallel_for(
       &resources_.get().compute_tp(), 0, uris.size(), [&](size_t i) {
-        RETURN_NOT_OK(resources_.get().vfs().remove_dir(uris[i]));
+        auto& vfs = resources_.get().vfs();
+        throw_if_not_ok(vfs.remove_dir(uris[i]));
         bool is_file = false;
-        RETURN_NOT_OK(
-            resources_.get().vfs().is_file(commit_uris_to_delete[i], &is_file));
+        throw_if_not_ok(vfs.is_file(commit_uris_to_delete[i], &is_file));
         if (is_file) {
-          RETURN_NOT_OK(
-              resources_.get().vfs().remove_file(commit_uris_to_delete[i]));
+          throw_if_not_ok(vfs.remove_file(commit_uris_to_delete[i]));
         }
         return Status::Ok();
       }));
@@ -583,9 +585,7 @@ const std::set<std::string>& ArrayDirectory::dir_names() {
 }
 
 std::vector<URI> ArrayDirectory::ls(const URI& uri) const {
-  auto&& [st, opt_dir_entries] = resources_.get().vfs().ls_with_sizes(uri);
-  throw_if_not_ok(st);
-  auto dir_entries = opt_dir_entries.value();
+  auto dir_entries = resources_.get().vfs().ls_with_sizes(uri);
   auto dirs = dir_names();
   std::vector<URI> uris;
 
@@ -939,7 +939,7 @@ ArrayDirectory::compute_fragment_uris_v1_v11(
     if (stdx::string::starts_with(array_dir_uris[i].last_path_part(), "."))
       return Status::Ok();
     int32_t flag;
-    RETURN_NOT_OK(this->is_fragment(
+    throw_if_not_ok(this->is_fragment(
         array_dir_uris[i], ok_uris, consolidated_commit_uris_set_, &flag));
     is_fragment[i] = (uint8_t)flag;
     return Status::Ok();
@@ -1078,11 +1078,11 @@ ArrayDirectory::compute_uris_to_vacuum(
   auto& tp = resources_.get().compute_tp();
   auto status = parallel_for(&tp, 0, vac_files.size(), [&](size_t i) {
     uint64_t size = 0;
-    RETURN_NOT_OK(resources_.get().vfs().file_size(vac_files[i], &size));
+    auto& vfs = resources_.get().vfs();
+    throw_if_not_ok(vfs.file_size(vac_files[i], &size));
     std::string names;
     names.resize(size);
-    RETURN_NOT_OK(
-        resources_.get().vfs().read(vac_files[i], 0, &names[0], size));
+    throw_if_not_ok(vfs.read(vac_files[i], 0, &names[0], size));
     std::stringstream ss(names);
     bool vacuum_vac_file = true;
     for (std::string uri_str; std::getline(ss, uri_str);) {
@@ -1229,7 +1229,6 @@ URI ArrayDirectory::select_latest_array_schema_uri() {
   }
 
   optional<URI> latest_uri = nullopt;
-  uint64_t latest_ts = 0;
 
   for (auto& uri : array_schema_uris_) {
     FragmentID fragment_id{uri};
@@ -1239,9 +1238,8 @@ URI ArrayDirectory::select_latest_array_schema_uri() {
     }
 
     auto ts_range{fragment_id.timestamp_range()};
-    if (ts_range.second > latest_ts && ts_range.second <= timestamp_end_) {
+    if (ts_range.second <= timestamp_end_) {
       latest_uri = uri;
-      latest_ts = ts_range.second;
     }
   }
 
