@@ -5,7 +5,7 @@
  *
  * The MIT License
  *
- * @copyright Copyright (c) 2017-2022 TileDB, Inc.
+ * @copyright Copyright (c) 2017-2024 TileDB, Inc.
  * @copyright Copyright (c) 2016 MIT and Intel Corporation
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -39,6 +39,8 @@
 #include "tiledb/common/common.h"
 #include "tiledb/common/pmr.h"
 #include "tiledb/common/status.h"
+#include "tiledb/sm/array_schema/current_domain.h"
+#include "tiledb/sm/array_schema/domain.h"
 #include "tiledb/sm/filesystem/uri.h"
 #include "tiledb/sm/filter/filter_pipeline.h"
 #include "tiledb/sm/misc/constants.h"
@@ -46,8 +48,7 @@
 
 using namespace tiledb::common;
 
-namespace tiledb {
-namespace sm {
+namespace tiledb::sm {
 
 class Attribute;
 class Buffer;
@@ -57,6 +58,7 @@ class DimensionLabel;
 class Domain;
 class Enumeration;
 class MemoryTracker;
+class CurrentDomain;
 
 enum class ArrayType : uint8_t;
 enum class Compressor : uint8_t;
@@ -121,6 +123,7 @@ class ArraySchema {
    * @param cell_validity_filters
    *    The filter pipeline run on validity tiles for nullable attributes.
    * @param coords_filters The filter pipeline run on coordinate tiles.
+   * @param current_domain The array current domain object
    * @param memory_tracker The memory tracker of the array this fragment
    *     metadata corresponds to.
    **/
@@ -142,6 +145,7 @@ class ArraySchema {
       FilterPipeline cell_var_offsets_filters,
       FilterPipeline cell_validity_filters,
       FilterPipeline coords_filters,
+      shared_ptr<CurrentDomain> current_domain,
       shared_ptr<MemoryTracker> memory_tracker);
 
   /**
@@ -212,7 +216,10 @@ class ArraySchema {
   }
 
   /** Returns the attributes. */
-  const tdb::pmr::vector<shared_ptr<const Attribute>>& attributes() const;
+  inline const tdb::pmr::vector<shared_ptr<const Attribute>>& attributes()
+      const {
+    return attributes_;
+  }
 
   /** Returns the capacity. */
   uint64_t capacity() const;
@@ -263,6 +270,11 @@ class ArraySchema {
   /** Return the pipeline used for coordinates. */
   const FilterPipeline& coords_filters() const;
 
+  /** Return the array current domain. */
+  inline const CurrentDomain& current_domain() const {
+    return *current_domain_;
+  }
+
   /** True if the array is dense. */
   bool dense() const;
 
@@ -297,9 +309,6 @@ class ArraySchema {
   /** Returns the number of dimensions. */
   dimension_size_type dim_num() const;
 
-  /** Dumps the array schema in ASCII format in the selected output. */
-  void dump(FILE* out) const;
-
   /**
    * Checks if the array schema has a attribute of the given name.
    *
@@ -327,14 +336,6 @@ class ArraySchema {
   /** Returns true if the input name is nullable. */
   bool is_nullable(const std::string& name) const;
 
-  /**
-   * Serializes the array schema object into a buffer.
-   *
-   * @param serializer The object the array schema is serialized into.
-   * @return Status
-   */
-  void serialize(Serializer& serializer) const;
-
   /** Returns the tile order. */
   Layout tile_order() const;
 
@@ -431,6 +432,13 @@ class ArraySchema {
    */
   std::vector<std::string> get_enumeration_names() const;
 
+  /**
+   * Get a vector of loaded Enumerations.
+   *
+   * @return A vector of loaded Enumerations.
+   */
+  std::vector<shared_ptr<const Enumeration>> get_loaded_enumerations() const;
+
   /**
    * Get a vector of loaded Enumeration names.
    *
@@ -569,9 +577,12 @@ class ArraySchema {
   /** Returns the timestamp range. */
   std::pair<uint64_t, uint64_t> timestamp_range() const;
 
-  /** Returns the the first timestamp. */
+  /** Returns the first timestamp. */
   uint64_t timestamp_start() const;
 
+  /** Returns the end timestamp */
+  uint64_t timestamp_end() const;
+
   /** Returns the array schema uri. */
   const URI& uri() const;
 
@@ -586,6 +597,42 @@ class ArraySchema {
       std::optional<std::pair<uint64_t, uint64_t>> timestamp_range =
           std::nullopt);
 
+  /** Returns the enumeration map. */
+  inline const tdb::pmr::
+      unordered_map<std::string, shared_ptr<const Enumeration>>&
+      enumeration_map() const {
+    return enumeration_map_;
+  }
+
+  /** Returns the enumeration path map. */
+  inline const tdb::pmr::unordered_map<std::string, std::string>&
+  enumeration_path_map() const {
+    return enumeration_path_map_;
+  }
+
+  /** Returns the dimension labels. */
+  inline const tdb::pmr::vector<shared_ptr<const DimensionLabel>>&
+  dimension_labels() const {
+    return dimension_labels_;
+  }
+
+  /**
+   * Expand the array current domain
+   *
+   * @param new_current_domain The new array current domain we want to expand to
+   */
+  void expand_current_domain(shared_ptr<CurrentDomain> new_current_domain);
+
+  /**
+   * Set the array current domain on the schema
+   *
+   * @param current_domain The array current domain we want to set on the schema
+   */
+  void set_current_domain(shared_ptr<CurrentDomain> current_domain);
+
+  /** Array current domain accessor */
+  shared_ptr<CurrentDomain> get_current_domain() const;
+
  private:
   /* ********************************* */
   /*         PRIVATE ATTRIBUTES        */
@@ -689,7 +736,7 @@ class ArraySchema {
   tdb::pmr::unordered_map<std::string, shared_ptr<const Enumeration>>
       enumeration_map_;
 
-  /** A map of Enumeration names to Enumeration URIs */
+  /** A map of Enumeration names to Enumeration filenames */
   tdb::pmr::unordered_map<std::string, std::string> enumeration_path_map_;
 
   /** The filter pipeline run on offset tiles for var-length attributes. */
@@ -701,6 +748,9 @@ class ArraySchema {
   /** The filter pipeline run on coordinate tiles. */
   FilterPipeline coords_filters_;
 
+  /** The array current domain */
+  shared_ptr<CurrentDomain> current_domain_;
+
   /** Mutex for thread-safety. */
   mutable std::mutex mtx_;
 
@@ -744,7 +794,10 @@ class ArraySchema {
   void clear();
 };
 
-}  // namespace sm
-}  // namespace tiledb
+}  // namespace tiledb::sm
+
+/** Converts the filter into a string representation. */
+std::ostream& operator<<(
+    std::ostream& os, const tiledb::sm::ArraySchema& schema);
 
 #endif  // TILEDB_ARRAY_SCHEMA_H
