@@ -40,6 +40,7 @@
 #include "tiledb/sm/array/array.h"
 #include "tiledb/sm/consolidator/consolidator.h"
 #include "tiledb/sm/misc/types.h"
+#include "tiledb/sm/storage_manager/context_resources.h"
 #include "tiledb/sm/storage_manager/storage_manager_declaration.h"
 
 #include <vector>
@@ -79,8 +80,14 @@ struct FragmentConsolidationConfig : Consolidator::ConsolidationConfigBase {
   float amplification_;
   /** Attribute buffer size. */
   uint64_t buffer_size_;
-  /** Total buffer size for all attributes. */
-  uint64_t total_buffer_size_;
+  /** Total memory budget for consolidation operation. */
+  uint64_t total_budget_;
+  /** Consolidation buffers weight used to partition total budget. */
+  uint64_t buffers_weight_;
+  /** Reader weight used to partition total budget. */
+  uint64_t reader_weight_;
+  /** Writer weight used to partition total budget. */
+  uint64_t writer_weight_;
   /** Max fragment size. */
   uint64_t max_fragment_size_;
   /**
@@ -124,13 +131,15 @@ class FragmentConsolidationWorkspace {
    * @param config The consolidation config.
    * @param array_schema The array schema.
    * @param avg_cell_sizes The average cell sizes.
+   * @param total_buffers_budget Total budget for the consolidation buffers.
    * @return a consolidation workspace containing the buffers
    */
   void resize_buffers(
       stats::Stats* stats,
       const FragmentConsolidationConfig& config,
       const ArraySchema& array_schema,
-      std::unordered_map<std::string, uint64_t>& avg_cell_sizes);
+      std::unordered_map<std::string, uint64_t>& avg_cell_sizes,
+      uint64_t total_buffers_budget);
 
   /** Accessor for buffers. */
   tdb::pmr::vector<span<std::byte>>& buffers() {
@@ -165,11 +174,21 @@ class FragmentConsolidator : public Consolidator {
   /**
    * Constructor.
    *
+   * This is a transitional constructor in the sense that we are working
+   * on removing the dependency of all Consolidator classes on StorageManager.
+   * For now we still need to keep the storage_manager argument, but once the
+   * dependency is gone the signature will be
+   * FragmentConsolidator(ContextResources&, const Config&).
+   *
+   * @param resources The context resources.
    * @param config Config.
-   * @param storage_manager Storage manager.
+   * @param storage_manager A StorageManager pointer.
+   *    (this will go away in the near future)
    */
   explicit FragmentConsolidator(
-      const Config& config, StorageManager* storage_manager);
+      ContextResources& resources,
+      const Config& config,
+      StorageManager* storage_manager);
 
   /** Destructor. */
   ~FragmentConsolidator() = default;
@@ -310,15 +329,19 @@ class FragmentConsolidator : public Consolidator {
    * @param query_r This query reads from the fragments to be consolidated.
    * @param query_w This query writes to the new consolidated fragment.
    * @param new_fragment_uri The URI of the new fragment to be created.
+   * @param read_memory_budget Memory budget for the read operation.
+   * @param write_memory_budget Memory budget for the write operation.
    * @return Status
    */
   Status create_queries(
       shared_ptr<Array> array_for_reads,
       shared_ptr<Array> array_for_writes,
       const NDRange& subarray,
-      Query** query_r,
-      Query** query_w,
-      URI* new_fragment_uri);
+      tdb_unique_ptr<Query>& query_r,
+      tdb_unique_ptr<Query>& query_w,
+      URI* new_fragment_uri,
+      uint64_t read_memory_budget,
+      uint64_t write_memory_budget);
 
   /**
    * Based on the input fragment info, this algorithm decides the (sorted) list
