@@ -5,7 +5,7 @@
  *
  * The MIT License
  *
- * @copyright Copyright (c) 2017-2021 TileDB, Inc.
+ * @copyright Copyright (c) 2017-2024 TileDB, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -49,18 +49,17 @@
 #include "tiledb/common/common.h"
 #include "tiledb/common/filesystem/directory_entry.h"
 #include "tiledb/common/logger.h"
+#include "tiledb/common/stdx_string.h"
 #include "tiledb/common/unique_rwlock.h"
 #include "tiledb/platform/cert_file.h"
 #include "tiledb/sm/filesystem/gcs.h"
 #include "tiledb/sm/misc/parallel_functions.h"
 #include "tiledb/sm/misc/tdb_math.h"
-#include "tiledb/sm/misc/utils.h"
 
 using namespace tiledb::common;
 using tiledb::common::filesystem::directory_entry;
 
-namespace tiledb {
-namespace sm {
+namespace tiledb::sm {
 
 /* ********************************* */
 /*     CONSTRUCTORS & DESTRUCTORS    */
@@ -438,7 +437,7 @@ Status GCS::remove_dir(const URI& uri) const {
   std::vector<std::string> paths;
   RETURN_NOT_OK(ls(uri, &paths, ""));
   auto status = parallel_for(thread_pool_, 0, paths.size(), [&](size_t i) {
-    RETURN_NOT_OK(remove_object(URI(paths[i])));
+    throw_if_not_ok(remove_object(URI(paths[i])));
     return Status::Ok();
   });
   RETURN_NOT_OK(status);
@@ -476,32 +475,27 @@ Status GCS::ls(
     const std::string& delimiter,
     const int max_paths) const {
   assert(paths);
-  auto&& [st, entries] = ls_with_sizes(uri, delimiter, max_paths);
-  RETURN_NOT_OK(st);
 
-  for (auto& fs : *entries) {
+  for (auto& fs : ls_with_sizes(uri, delimiter, max_paths)) {
     paths->emplace_back(fs.path().native());
   }
 
   return Status::Ok();
 }
 
-tuple<Status, optional<std::vector<directory_entry>>> GCS::ls_with_sizes(
+std::vector<directory_entry> GCS::ls_with_sizes(
     const URI& uri, const std::string& delimiter, int max_paths) const {
-  RETURN_NOT_OK_TUPLE(init_client(), nullopt);
+  throw_if_not_ok(init_client());
 
   const URI uri_dir = uri.add_trailing_slash();
 
   if (!uri_dir.is_gcs()) {
-    auto st = LOG_STATUS(Status_GCSError(
-        std::string("URI is not a GCS URI: " + uri_dir.to_string())));
-    return {st, nullopt};
+    throw GCSException("URI is not a GCS URI: " + uri_dir.to_string());
   }
 
   std::string bucket_name;
   std::string object_path;
-  RETURN_NOT_OK_TUPLE(
-      parse_gcs_uri(uri_dir, &bucket_name, &object_path), nullopt);
+  throw_if_not_ok(parse_gcs_uri(uri_dir, &bucket_name, &object_path));
 
   std::vector<directory_entry> entries;
   google::cloud::storage::Prefix prefix_option(object_path);
@@ -514,10 +508,9 @@ tuple<Status, optional<std::vector<directory_entry>>> GCS::ls_with_sizes(
     if (!object_metadata.ok()) {
       const google::cloud::Status status = object_metadata.status();
 
-      auto st = LOG_STATUS(Status_GCSError(std::string(
+      throw GCSException(
           "List objects failed on: " + uri.to_string() + " (" +
-          status.message() + ")")));
-      return {st, nullopt};
+          status.message() + ")");
     }
 
     if (entries.size() >= static_cast<size_t>(max_paths)) {
@@ -548,7 +541,84 @@ tuple<Status, optional<std::vector<directory_entry>>> GCS::ls_with_sizes(
     }
   }
 
-  return {Status::Ok(), entries};
+  return entries;
+}
+
+LsObjects GCS::ls_filtered_impl(
+    const URI& uri,
+    std::function<bool(const std::string_view, uint64_t)> file_filter,
+    bool recursive) const {
+  throw_if_not_ok(init_client());
+
+  const URI uri_dir = uri.add_trailing_slash();
+
+  if (!uri_dir.is_gcs()) {
+    throw GCSException(
+        std::string("URI is not a GCS URI: " + uri_dir.to_string()));
+  }
+
+  std::string prefix = uri_dir.backend_name() + "://";
+  std::string bucket_name;
+  std::string object_path;
+  throw_if_not_ok(parse_gcs_uri(uri_dir, &bucket_name, &object_path));
+
+  LsObjects result;
+
+  auto to_directory_entry =
+      [&bucket_name, &prefix](const google::cloud::storage::ObjectMetadata& obj)
+      -> LsObjects::value_type {
+    return {
+        prefix + bucket_name + "/" +
+            remove_front_slash(remove_trailing_slash(obj.name())),
+        obj.size()};
+  };
+
+  if (recursive) {
+    auto it = client_->ListObjects(
+        bucket_name, google::cloud::storage::Prefix(std::move(object_path)));
+    for (const auto& object_metadata : it) {
+      if (!object_metadata) {
+        throw GCSException(std::string(
+            "List objects failed on: " + uri.to_string() + " (" +
+            object_metadata.status().message() + ")"));
+      }
+
+      auto entry = to_directory_entry(*object_metadata);
+      if (file_filter(entry.first, entry.second)) {
+        result.emplace_back(std::move(entry));
+      }
+    }
+  } else {
+    auto it = client_->ListObjectsAndPrefixes(
+        bucket_name,
+        google::cloud::storage::Prefix(std::move(object_path)),
+        google::cloud::storage::Delimiter("/"));
+    for (const auto& object_metadata : it) {
+      if (!object_metadata) {
+        throw GCSException(std::string(
+            "List objects failed on: " + uri.to_string() + " (" +
+            object_metadata.status().message() + ")"));
+      }
+
+      LsObjects::value_type entry;
+      if (absl::holds_alternative<google::cloud::storage::ObjectMetadata>(
+              *object_metadata)) {
+        entry = to_directory_entry(
+            absl::get<google::cloud::storage::ObjectMetadata>(
+                *object_metadata));
+      } else {
+        entry = {
+            prefix + bucket_name + "/" +
+                absl::get<std::string>(*object_metadata),
+            0};
+      }
+      if (file_filter(entry.first, entry.second)) {
+        result.push_back(std::move(entry));
+      }
+    }
+  }
+
+  return result;
 }
 
 Status GCS::move_object(const URI& old_uri, const URI& new_uri) {
@@ -1307,7 +1377,6 @@ Status GCS::parse_gcs_uri(
   return Status::Ok();
 }
 
-}  // namespace sm
-}  // namespace tiledb
+}  // namespace tiledb::sm
 
 #endif
