@@ -75,6 +75,10 @@
 #include "tiledb/sm/filesystem/azure.h"
 #endif  // HAVE_AZURE
 
+namespace tiledb::common {
+class LogDurationInstrument;
+}
+
 using namespace tiledb::common;
 using tiledb::common::filesystem::directory_entry;
 
@@ -173,6 +177,8 @@ struct VFSParameters {
       , read_ahead_cache_size_(
             config.get<uint64_t>("vfs.read_ahead_cache_size").value())
       , read_ahead_size_(config.get<uint64_t>("vfs.read_ahead_size").value())
+      , log_operations_(
+            config.get<bool>("vfs.log_operations", Config::must_find))
       , read_logging_mode_(ReadLoggingMode::DISABLED) {
     auto log_mode = config.get<std::string>("vfs.read_logging_mode").value();
     if (log_mode == "") {
@@ -208,6 +214,9 @@ struct VFSParameters {
   /** The byte size to read-ahead for each read. */
   uint64_t read_ahead_size_;
 
+  /** Whether to log all VFS operations. */
+  bool log_operations_;
+
   /** The read logging mode to use. */
   ReadLoggingMode read_logging_mode_;
 };
@@ -306,11 +315,13 @@ class VFS : private VFSBase, protected S3_within_VFS {
 
   /** Constructor.
    * @param parent_stats The parent stats to inherit from.
+   * @param logger The logger to use. Optional, can be nullptr.
    * @param compute_tp Thread pool for compute-bound tasks.
    * @param io_tp Thread pool for io-bound tasks.
    * @param config Configuration parameters.
    **/
   VFS(stats::Stats* parent_stats,
+      Logger* logger,
       ThreadPool* compute_tp,
       ThreadPool* io_tp,
       const Config& config);
@@ -410,6 +421,16 @@ class VFS : private VFSBase, protected S3_within_VFS {
    */
   Status remove_dir(const URI& uri) const;
 
+  /**
+   * Removes a given empty directory. No exceptions are raised if the directory
+   * was not empty.
+   *
+   * This function does nothing when the URI points to an object store.
+   *
+   * @param uri The uri of the directory to be removed
+   */
+  void remove_dir_if_empty(const URI& uri) const;
+
   /**
    * Deletes directories in parallel from the given vector of directories.
    *
@@ -506,55 +527,52 @@ class VFS : private VFSBase, protected S3_within_VFS {
    * @param parent The target directory to list.
    * @return All entries that are contained in the parent
    */
-  tuple<Status, optional<std::vector<directory_entry>>> ls_with_sizes(
-      const URI& parent) const;
+  std::vector<directory_entry> ls_with_sizes(const URI& parent) const;
 
   /**
    * Lists objects and object information that start with `prefix`, invoking
    * the FilePredicate on each entry collected and the DirectoryPredicate on
    * common prefixes for pruning.
    *
-   * Currently this API is only supported for Posix and S3.
+   * Currently this API is only supported for local files, S3 and Azure.
    *
    * @param parent The parent prefix to list sub-paths.
    * @param f The FilePredicate to invoke on each object for filtering.
    * @param d The DirectoryPredicate to invoke on each common prefix for
    *    pruning. This is currently unused, but is kept here for future support.
+   * @param recursive Whether to list the objects recursively.
    * @return Vector of results with each entry being a pair of the string URI
    *    and object size.
    */
   template <FilePredicate F, DirectoryPredicate D = DirectoryFilter>
-  LsObjects ls_recursive(
+  LsObjects ls_filtered(
       const URI& parent,
       [[maybe_unused]] F f,
-      [[maybe_unused]] D d = accept_all_dirs) const {
+      [[maybe_unused]] D d,
+      bool recursive) const {
     LsObjects results;
     try {
       if (parent.is_file()) {
 #ifdef _WIN32
-        results = win_.ls_filtered(parent, f, d, true);
+        results = win_.ls_filtered(parent, f, d, recursive);
 #else
-        results = posix_.ls_filtered(parent, f, d, true);
+        results = posix_.ls_filtered(parent, f, d, recursive);
 #endif
       } else if (parent.is_s3()) {
 #ifdef HAVE_S3
-        results = s3().ls_filtered(parent, f, d, true);
+        results = s3().ls_filtered(parent, f, d, recursive);
 #else
         throw filesystem::VFSException("TileDB was built without S3 support");
 #endif
       } else if (parent.is_gcs()) {
 #ifdef HAVE_GCS
-        throw filesystem::VFSException(
-            "Recursive ls over " + parent.backend_name() +
-            " storage backend is not supported.");
+        results = gcs_.ls_filtered(parent, f, d, recursive);
 #else
         throw filesystem::VFSException("TileDB was built without GCS support");
 #endif
       } else if (parent.is_azure()) {
 #ifdef HAVE_AZURE
-        throw filesystem::VFSException(
-            "Recursive ls over " + parent.backend_name() +
-            " storage backend is not supported.");
+        results = azure_.ls_filtered(parent, f, d, recursive);
 #else
         throw filesystem::VFSException(
             "TileDB was built without Azure support");
@@ -581,6 +599,28 @@ class VFS : private VFSBase, protected S3_within_VFS {
     return results;
   }
 
+  /**
+   * Recursively lists objects and object information that start with `prefix`,
+   * invoking the FilePredicate on each entry collected and the
+   * DirectoryPredicate on common prefixes for pruning.
+   *
+   * Currently this API is only supported for local files, S3, Azure and GCS.
+   *
+   * @param parent The parent prefix to list sub-paths.
+   * @param f The FilePredicate to invoke on each object for filtering.
+   * @param d The DirectoryPredicate to invoke on each common prefix for
+   *    pruning. This is currently unused, but is kept here for future support.
+   * @return Vector of results with each entry being a pair of the string URI
+   *    and object size.
+   */
+  template <FilePredicate F, DirectoryPredicate D = DirectoryFilter>
+  LsObjects ls_recursive(
+      const URI& parent,
+      [[maybe_unused]] F f,
+      [[maybe_unused]] D d = accept_all_dirs) const {
+    return ls_filtered(parent, f, d, true);
+  }
+
   /**
    * Renames a file.
    *
@@ -881,8 +921,9 @@ class VFS : private VFSBase, protected S3_within_VFS {
 
       const uint64_t size = buffer.size();
       ReadAheadBuffer ra_buffer(offset, std::move(buffer));
-      return LRUCache<std::string, ReadAheadBuffer>::insert(
+      LRUCache<std::string, ReadAheadBuffer>::insert(
           uri.to_string(), std::move(ra_buffer), size);
+      return Status::Ok();
     }
 
    private:
@@ -928,6 +969,9 @@ class VFS : private VFSBase, protected S3_within_VFS {
    **/
   Config config_;
 
+  /** Logger. */
+  Logger* logger_;
+
   /** The set with the supported filesystems. */
   std::set<Filesystem> supported_fs_;
 
@@ -1005,6 +1049,28 @@ class VFS : private VFSBase, protected S3_within_VFS {
    * @param nbytes The number of bytes requested.
    */
   void log_read(const URI& uri, uint64_t offset, uint64_t nbytes);
+
+  /**
+   * Creates a LogDurationInstrument, if enabled in config.
+   *
+   * @param uri The URI of the object being operated on.
+   * @param operation_name The name of the operation. Defaults to the name of
+   * the calling function.
+   */
+  optional<LogDurationInstrument> make_log_duration_instrument(
+      const URI& uri, const std::string_view operation_name) const;
+
+  /**
+   * Creates a LogDurationInstrument, if enabled in config.
+   *
+   * @param uri The URI of the object being operated on.
+   * @param operation_name The name of the operation. Defaults to the name of
+   * the calling function.
+   */
+  optional<LogDurationInstrument> make_log_duration_instrument(
+      const URI& source,
+      const URI& destination,
+      const std::string_view operation_name) const;
 };
 
 }  // namespace tiledb::sm
