@@ -5,7 +5,7 @@
  *
  * The MIT License
  *
- * @copyright Copyright (c) 2017-2021 TileDB, Inc.
+ * @copyright Copyright (c) 2017-2024 TileDB, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -45,6 +45,7 @@
 #include <sstream>
 #include <string_view>
 
+#include "filesystem_base.h"
 #include "path_win.h"
 #include "tiledb/common/common.h"
 #include "tiledb/common/filesystem/directory_entry.h"
@@ -54,15 +55,13 @@
 #include "tiledb/common/stdx_string.h"
 #include "tiledb/sm/misc/constants.h"
 #include "tiledb/sm/misc/tdb_math.h"
-#include "tiledb/sm/misc/utils.h"
 #include "uri.h"
 #include "win.h"
 
 using namespace tiledb::common;
 using tiledb::common::filesystem::directory_entry;
 
-namespace tiledb {
-namespace sm {
+namespace tiledb::sm {
 
 namespace {
 /** Returns the last Windows error message string. */
@@ -284,6 +283,19 @@ Status Win::remove_dir(const std::string& path) const {
   }
 }
 
+bool Win::remove_dir_if_empty(const std::string& path) const {
+  if (!RemoveDirectoryA(path.c_str())) {
+    auto gle = GetLastError();
+    if (gle == ERROR_DIR_NOT_EMPTY) {
+      return false;
+    }
+    throw IOError(
+        std::string("Failed to delete directory '") + path + "' " +
+        get_last_error_msg(gle, "RemoveDirectory"));
+  }
+  return true;
+}
+
 Status Win::remove_file(const std::string& path) const {
   if (!DeleteFile(path.c_str())) {
     return LOG_STATUS(Status_IOError(std::string(
@@ -326,7 +338,7 @@ Status Win::init(const Config& config) {
 }
 
 bool Win::is_dir(const std::string& path) const {
-  return PathIsDirectory(path.c_str());
+  return PathFileExists(path.c_str()) && PathIsDirectory(path.c_str());
 }
 
 bool Win::is_file(const std::string& path) const {
@@ -334,18 +346,14 @@ bool Win::is_file(const std::string& path) const {
 }
 
 Status Win::ls(const std::string& path, std::vector<std::string>* paths) const {
-  auto&& [st, entries] = ls_with_sizes(URI(path));
-  RETURN_NOT_OK(st);
-
-  for (auto& fs : *entries) {
+  for (auto& fs : ls_with_sizes(URI(path))) {
     paths->emplace_back(fs.path().native());
   }
 
   return Status::Ok();
 }
 
-tuple<Status, optional<std::vector<directory_entry>>> Win::ls_with_sizes(
-    const URI& uri) const {
+std::vector<directory_entry> Win::ls_with_sizes(const URI& uri) const {
   auto path = uri.to_path();
   bool ends_with_slash = path.length() > 0 && path[path.length() - 1] == '\\';
   const std::string glob = path + (ends_with_slash ? "*" : "\\*");
@@ -389,7 +397,7 @@ tuple<Status, optional<std::vector<directory_entry>>> Win::ls_with_sizes(
   }
 
   FindClose(find_h);
-  return {Status::Ok(), entries};
+  return entries;
 
 err:
   auto gle = GetLastError();
@@ -401,11 +409,9 @@ err:
     offender.append(" ");
     offender.append(what_call);
   }
-  std::string errmsg(
+  throw IOError(
       "Failed to list directory \"" + path + "\" " +
       get_last_error_msg(gle, offender.c_str()));
-  auto st = LOG_STATUS(Status_IOError(errmsg));
-  return {st, nullopt};
 }
 
 Status Win::move_path(
@@ -593,7 +599,6 @@ Status Win::write_at(
   return Status::Ok();
 }
 
-}  // namespace sm
-}  // namespace tiledb
+}  // namespace tiledb::sm
 
 #endif  // _WIN32
