@@ -42,12 +42,18 @@
 #include "tiledb/sm/fragment/fragment_identifier.h"
 #include "tiledb/sm/misc/parallel_functions.h"
 #include "tiledb/sm/misc/tdb_time.h"
-#include "tiledb/sm/misc/utils.h"
 #include "tiledb/sm/rest/rest_client.h"
 #include "tiledb/sm/tile/generic_tile_io.h"
 
 namespace tiledb::sm {
 
+class FragmentInfoException : public StatusException {
+ public:
+  explicit FragmentInfoException(const std::string& message)
+      : StatusException("FragmentInfo", message) {
+  }
+};
+
 /* ****************************** */
 /*   CONSTRUCTORS & DESTRUCTORS   */
 /* ****************************** */
@@ -67,8 +73,7 @@ FragmentInfo::~FragmentInfo() = default;
 
 void FragmentInfo::set_config(const Config& config) {
   if (loaded_) {
-    throw StatusException(
-        Status_FragmentInfoError("[set_config] Cannot set config after load"));
+    throw FragmentInfoException("[set_config] Cannot set config after load");
   }
   config_.inherit(config);
 }
@@ -78,44 +83,17 @@ void FragmentInfo::expand_anterior_ndrange(
   domain.expand_ndrange(range, &anterior_ndrange_);
 }
 
-void FragmentInfo::dump(FILE* out) const {
-  ensure_loaded();
-  if (out == nullptr)
-    out = stdout;
-
-  auto fragment_num = this->fragment_num();
-
-  std::stringstream ss;
-  ss << "- Fragment num: " << fragment_num << "\n";
-  ss << "- Unconsolidated metadata num: " << unconsolidated_metadata_num_
-     << "\n";
-  ss << "- To vacuum num: " << to_vacuum_.size() << "\n";
-
-  if (!to_vacuum_.empty()) {
-    ss << "- To vacuum URIs:\n";
-    for (const auto& v : to_vacuum_)
-      ss << "  > " << v.c_str() << "\n";
-  }
-
-  fprintf(out, "%s", ss.str().c_str());
-
-  for (uint32_t fid = 0; fid < fragment_num; ++fid) {
-    auto meta = single_fragment_info_vec_[fid].meta();
-    auto dim_types = meta->dim_types();
-    fprintf(out, "- Fragment #%u:\n", fid + 1);
-    single_fragment_info_vec_[fid].dump(dim_types, out);
-  }
-}
-
 Status FragmentInfo::get_dense(uint32_t fid, int32_t* dense) const {
   ensure_loaded();
-  if (dense == nullptr)
-    return LOG_STATUS(Status_FragmentInfoError(
-        "Cannot check if fragment is dense; Dense argument cannot be null"));
+  if (dense == nullptr) {
+    throw FragmentInfoException(
+        "Cannot check if fragment is dense; Dense argument cannot be null");
+  }
 
-  if (fid >= fragment_num())
-    return LOG_STATUS(Status_FragmentInfoError(
-        "Cannot check if fragment is dense; Invalid fragment index"));
+  if (fid >= fragment_num()) {
+    throw FragmentInfoException(
+        "Cannot check if fragment is dense; Invalid fragment index");
+  }
 
   *dense = (int32_t)!single_fragment_info_vec_[fid].sparse();
 
@@ -124,13 +102,15 @@ Status FragmentInfo::get_dense(uint32_t fid, int32_t* dense) const {
 
 Status FragmentInfo::get_sparse(uint32_t fid, int32_t* sparse) const {
   ensure_loaded();
-  if (sparse == nullptr)
-    return LOG_STATUS(Status_FragmentInfoError(
-        "Cannot check if fragment is sparse; Sparse argument cannot be null"));
+  if (sparse == nullptr) {
+    throw FragmentInfoException(
+        "Cannot check if fragment is sparse; Sparse argument cannot be null");
+  }
 
-  if (fid >= fragment_num())
-    return LOG_STATUS(Status_FragmentInfoError(
-        "Cannot check if fragment is sparse; Invalid fragment index"));
+  if (fid >= fragment_num()) {
+    throw FragmentInfoException(
+        "Cannot check if fragment is sparse; Invalid fragment index");
+  }
 
   *sparse = (int32_t)single_fragment_info_vec_[fid].sparse();
 
@@ -144,13 +124,15 @@ uint32_t FragmentInfo::fragment_num() const {
 
 Status FragmentInfo::get_cell_num(uint32_t fid, uint64_t* cell_num) const {
   ensure_loaded();
-  if (cell_num == nullptr)
-    return LOG_STATUS(Status_FragmentInfoError(
-        "Cannot get fragment URI; Cell number argument cannot be null"));
+  if (cell_num == nullptr) {
+    throw FragmentInfoException(
+        "Cannot get fragment URI; Cell number argument cannot be null");
+  }
 
-  if (fid >= fragment_num())
-    return LOG_STATUS(Status_FragmentInfoError(
-        "Cannot get fragment URI; Invalid fragment index"));
+  if (fid >= fragment_num()) {
+    throw FragmentInfoException(
+        "Cannot get fragment URI; Invalid fragment index");
+  }
 
   *cell_num = single_fragment_info_vec_[fid].cell_num();
 
@@ -159,9 +141,9 @@ Status FragmentInfo::get_cell_num(uint32_t fid, uint64_t* cell_num) const {
 
 Status FragmentInfo::get_total_cell_num(uint64_t* cell_num) const {
   ensure_loaded();
-  if (cell_num == nullptr)
-    return LOG_STATUS(
-        Status_FragmentInfoError("Cell number argument cannot be null"));
+  if (cell_num == nullptr) {
+    throw FragmentInfoException("Cell number argument cannot be null");
+  }
 
   // Return simple summation of cell counts in each fragment present without
   // any consideration of cells that may be overlapping, i.e. the count
@@ -182,7 +164,7 @@ Status FragmentInfo::get_total_cell_num(uint64_t* cell_num) const {
 const std::string& FragmentInfo::fragment_name(uint32_t fid) const {
   ensure_loaded();
   if (fid >= fragment_num())
-    throw Status_FragmentInfoError(
+    throw FragmentInfoException(
         "Cannot get fragment URI; Invalid fragment index");
 
   return single_fragment_info_vec_[fid].name();
@@ -190,13 +172,15 @@ const std::string& FragmentInfo::fragment_name(uint32_t fid) const {
 
 Status FragmentInfo::get_fragment_size(uint32_t fid, uint64_t* size) const {
   ensure_loaded();
-  if (size == nullptr)
-    return LOG_STATUS(Status_FragmentInfoError(
-        "Cannot get fragment URI; Size argument cannot be null"));
+  if (size == nullptr) {
+    throw FragmentInfoException(
+        "Cannot get fragment URI; Size argument cannot be null");
+  }
 
-  if (fid >= fragment_num())
-    return LOG_STATUS(Status_FragmentInfoError(
-        "Cannot get fragment URI; Invalid fragment index"));
+  if (fid >= fragment_num()) {
+    throw FragmentInfoException(
+        "Cannot get fragment URI; Invalid fragment index");
+  }
 
   *size = single_fragment_info_vec_[fid].fragment_size();
 
@@ -205,13 +189,15 @@ Status FragmentInfo::get_fragment_size(uint32_t fid, uint64_t* size) const {
 
 Status FragmentInfo::get_fragment_uri(uint32_t fid, const char** uri) const {
   ensure_loaded();
-  if (uri == nullptr)
-    return LOG_STATUS(Status_FragmentInfoError(
-        "Cannot get fragment URI; URI argument cannot be null"));
+  if (uri == nullptr) {
+    throw FragmentInfoException(
+        "Cannot get fragment URI; URI argument cannot be null");
+  }
 
-  if (fid >= fragment_num())
-    return LOG_STATUS(Status_FragmentInfoError(
-        "Cannot get fragment URI; Invalid fragment index"));
+  if (fid >= fragment_num()) {
+    throw FragmentInfoException(
+        "Cannot get fragment URI; Invalid fragment index");
+  }
 
   *uri = single_fragment_info_vec_[fid].uri().c_str();
 
@@ -220,13 +206,15 @@ Status FragmentInfo::get_fragment_uri(uint32_t fid, const char** uri) const {
 
 Status FragmentInfo::get_to_vacuum_uri(uint32_t fid, const char** uri) const {
   ensure_loaded();
-  if (uri == nullptr)
-    return LOG_STATUS(Status_FragmentInfoError(
-        "Cannot get URI of fragment to vacuum; URI argument cannot be null"));
+  if (uri == nullptr) {
+    throw FragmentInfoException(
+        "Cannot get URI of fragment to vacuum; URI argument cannot be null");
+  }
 
-  if (fid >= to_vacuum_.size())
-    return LOG_STATUS(Status_FragmentInfoError(
-        "Cannot get URI of fragment to vacuum; Invalid fragment index"));
+  if (fid >= to_vacuum_.size()) {
+    throw FragmentInfoException(
+        "Cannot get URI of fragment to vacuum; Invalid fragment index");
+  }
 
   *uri = to_vacuum_[fid].c_str();
 
@@ -236,17 +224,20 @@ Status FragmentInfo::get_to_vacuum_uri(uint32_t fid, const char** uri) const {
 Status FragmentInfo::get_timestamp_range(
     uint32_t fid, uint64_t* start, uint64_t* end) const {
   ensure_loaded();
-  if (start == nullptr)
-    return LOG_STATUS(Status_FragmentInfoError(
-        "Cannot get timestamp range; Start argument cannot be null"));
+  if (start == nullptr) {
+    throw FragmentInfoException(
+        "Cannot get timestamp range; Start argument cannot be null");
+  }
 
-  if (end == nullptr)
-    return LOG_STATUS(Status_FragmentInfoError(
-        "Cannot get timestamp range; End argument cannot be null"));
+  if (end == nullptr) {
+    throw FragmentInfoException(
+        "Cannot get timestamp range; End argument cannot be null");
+  }
 
-  if (fid >= fragment_num())
-    return LOG_STATUS(Status_FragmentInfoError(
-        "Cannot get fragment URI; Invalid fragment index"));
+  if (fid >= fragment_num()) {
+    throw FragmentInfoException(
+        "Cannot get fragment URI; Invalid fragment index");
+  }
 
   auto range = single_fragment_info_vec_[fid].timestamp_range();
   *start = range.first;
@@ -258,24 +249,28 @@ Status FragmentInfo::get_timestamp_range(
 Status FragmentInfo::get_non_empty_domain(
     uint32_t fid, uint32_t did, void* domain) const {
   ensure_loaded();
-  if (domain == nullptr)
-    return LOG_STATUS(Status_FragmentInfoError(
-        "Cannot get non-empty domain; Domain argument cannot be null"));
+  if (domain == nullptr) {
+    throw FragmentInfoException(
+        "Cannot get non-empty domain; Domain argument cannot be null");
+  }
 
-  if (fid >= fragment_num())
-    return LOG_STATUS(Status_FragmentInfoError(
-        "Cannot get non-empty domain; Invalid fragment index"));
+  if (fid >= fragment_num()) {
+    throw FragmentInfoException(
+        "Cannot get non-empty domain; Invalid fragment index");
+  }
 
   const auto& non_empty_domain =
       single_fragment_info_vec_[fid].non_empty_domain();
 
-  if (did >= non_empty_domain.size())
-    return LOG_STATUS(Status_FragmentInfoError(
-        "Cannot get non-empty domain; Invalid dimension index"));
+  if (did >= non_empty_domain.size()) {
+    throw FragmentInfoException(
+        "Cannot get non-empty domain; Invalid dimension index");
+  }
 
-  if (non_empty_domain[did].var_size())
-    return LOG_STATUS(Status_FragmentInfoError(
-        "Cannot get non-empty domain; Dimension is variable-sized"));
+  if (non_empty_domain[did].var_size()) {
+    throw FragmentInfoException(
+        "Cannot get non-empty domain; Dimension is variable-sized");
+  }
 
   assert(!non_empty_domain[did].empty());
   std::memcpy(
@@ -287,12 +282,14 @@ Status FragmentInfo::get_non_empty_domain(
 Status FragmentInfo::get_non_empty_domain(
     uint32_t fid, const char* dim_name, void* domain) const {
   ensure_loaded();
-  if (fid >= fragment_num())
-    return LOG_STATUS(Status_FragmentInfoError(
-        "Cannot get non-empty domain; Invalid fragment index"));
-  if (dim_name == nullptr)
-    return LOG_STATUS(Status_FragmentInfoError(
-        "Cannot get non-empty domain; Dimension name argument cannot be null"));
+  if (fid >= fragment_num()) {
+    throw FragmentInfoException(
+        "Cannot get non-empty domain; Invalid fragment index");
+  }
+  if (dim_name == nullptr) {
+    throw FragmentInfoException(
+        "Cannot get non-empty domain; Dimension name argument cannot be null");
+  }
 
   auto meta = single_fragment_info_vec_[fid].meta();
   const auto& array_schema = meta->array_schema();
@@ -306,10 +303,9 @@ Status FragmentInfo::get_non_empty_domain(
 
   // Dimension name not found
   if (did == dim_num) {
-    auto msg =
-        std::string("Cannot get non-empty domain; Invalid dimension name '") +
-        dim_name + "'";
-    return LOG_STATUS(Status_FragmentInfoError(msg));
+    throw FragmentInfoException(
+        "Cannot get non-empty domain; Invalid dimension name '" +
+        std::string(dim_name) + "'");
   }
 
   return get_non_empty_domain(fid, did, domain);
@@ -321,32 +317,34 @@ Status FragmentInfo::get_non_empty_domain_var_size(
     uint64_t* start_size,
     uint64_t* end_size) const {
   ensure_loaded();
-  if (start_size == nullptr)
-    return LOG_STATUS(
-        Status_FragmentInfoError("Cannot get non-empty domain var size; Start "
-                                 "size argument cannot be null"));
+  if (start_size == nullptr) {
+    throw FragmentInfoException(
+        "Cannot get non-empty domain var size; Start size cannot be null");
+  }
 
-  if (end_size == nullptr)
-    return LOG_STATUS(
-        Status_FragmentInfoError("Cannot get non-empty domain var size; End "
-                                 "size argument cannot be null"));
+  if (end_size == nullptr) {
+    throw FragmentInfoException(
+        "Cannot get non-empty domain var size; End size cannot be null");
+  }
 
-  if (fid >= fragment_num())
-    return LOG_STATUS(Status_FragmentInfoError(
-        "Cannot get non-empty domain var size; Invalid fragment index"));
+  if (fid >= fragment_num()) {
+    throw FragmentInfoException(
+        "Cannot get non-empty domain var size; Invalid fragment index");
+  }
 
   const auto& non_empty_domain =
       single_fragment_info_vec_[fid].non_empty_domain();
 
-  if (did >= non_empty_domain.size())
-    return LOG_STATUS(Status_FragmentInfoError(
-        "Cannot get non-empty domain var size; Invalid dimension index"));
+  if (did >= non_empty_domain.size()) {
+    throw FragmentInfoException(
+        "Cannot get non-empty domain var size; Invalid dimension index");
+  }
 
-  if (!non_empty_domain[did].var_size())
-    return LOG_STATUS(Status_FragmentInfoError(
-        "Cannot get non-empty domain var size; Dimension is fixed sized"));
+  if (!non_empty_domain[did].var_size()) {
+    throw FragmentInfoException(
+        "Cannot get non-empty domain var size; Dimension is fixed sized");
+  }
 
-  assert(!non_empty_domain[did].empty());
   *start_size = non_empty_domain[did].start_size();
   *end_size = non_empty_domain[did].end_size();
 
@@ -359,13 +357,15 @@ Status FragmentInfo::get_non_empty_domain_var_size(
     uint64_t* start_size,
     uint64_t* end_size) const {
   ensure_loaded();
-  if (fid >= fragment_num())
-    return LOG_STATUS(Status_FragmentInfoError(
-        "Cannot get var-sized non-empty domain; Invalid fragment index"));
-  if (dim_name == nullptr)
-    return LOG_STATUS(
-        Status_FragmentInfoError("Cannot get non-empty domain var size; "
-                                 "Dimension name argument cannot be null"));
+  if (fid >= fragment_num()) {
+    throw FragmentInfoException(
+        "Cannot get var-sized non-empty domain; Invalid fragment index");
+  }
+  if (dim_name == nullptr) {
+    throw FragmentInfoException(
+        "Cannot get non-empty domain var size; "
+        "Dimension name argument cannot be null");
+  }
 
   auto meta = single_fragment_info_vec_[fid].meta();
   const auto& array_schema = meta->array_schema();
@@ -379,11 +379,9 @@ Status FragmentInfo::get_non_empty_domain_var_size(
 
   // Dimension name not found
   if (did == dim_num) {
-    auto msg =
-        std::string(
-            "Cannot get non-empty domain var size; Invalid dimension name '") +
-        dim_name + "'";
-    return LOG_STATUS(Status_FragmentInfoError(msg));
+    throw FragmentInfoException(
+        "Cannot get non-empty domain var size; Invalid dimension name '" +
+        std::string(dim_name) + "'");
   }
 
   return get_non_empty_domain_var_size(fid, did, start_size, end_size);
@@ -392,31 +390,35 @@ Status FragmentInfo::get_non_empty_domain_var_size(
 Status FragmentInfo::get_non_empty_domain_var(
     uint32_t fid, uint32_t did, void* start, void* end) const {
   ensure_loaded();
-  if (start == nullptr)
-    return LOG_STATUS(
-        Status_FragmentInfoError("Cannot get non-empty domain var; Domain "
-                                 "start argument cannot be null"));
+  if (start == nullptr) {
+    throw FragmentInfoException(
+        "Cannot get non-empty domain var; Domain start argument cannot be "
+        "null");
+  }
 
-  if (end == nullptr)
-    return LOG_STATUS(Status_FragmentInfoError(
-        "Cannot get non-empty domain var; Domain end argument cannot be null"));
+  if (end == nullptr) {
+    throw FragmentInfoException(
+        "Cannot get non-empty domain var; Domain end argument cannot be null");
+  }
 
-  if (fid >= fragment_num())
-    return LOG_STATUS(Status_FragmentInfoError(
-        "Cannot get non-empty domain var; Invalid fragment index"));
+  if (fid >= fragment_num()) {
+    throw FragmentInfoException(
+        "Cannot get non-empty domain var; Invalid fragment index");
+  }
 
   const auto& non_empty_domain =
       single_fragment_info_vec_[fid].non_empty_domain();
 
-  if (did >= non_empty_domain.size())
-    return LOG_STATUS(Status_FragmentInfoError(
-        "Cannot get non-empty domain var; Invalid dimension index"));
+  if (did >= non_empty_domain.size()) {
+    throw FragmentInfoException(
+        "Cannot get non-empty domain var; Invalid dimension index");
+  }
 
-  if (!non_empty_domain[did].var_size())
-    return LOG_STATUS(Status_FragmentInfoError(
-        "Cannot get non-empty domain var; Dimension is fixed-sized"));
+  if (!non_empty_domain[did].var_size()) {
+    throw FragmentInfoException(
+        "Cannot get non-empty domain var; Dimension is fixed-sized");
+  }
 
-  assert(!non_empty_domain[did].empty());
   auto start_str = non_empty_domain[did].start_str();
   std::memcpy(start, start_str.data(), start_str.size());
   auto end_str = non_empty_domain[did].end_str();
@@ -428,13 +430,15 @@ Status FragmentInfo::get_non_empty_domain_var(
 Status FragmentInfo::get_non_empty_domain_var(
     uint32_t fid, const char* dim_name, void* start, void* end) const {
   ensure_loaded();
-  if (fid >= fragment_num())
-    return LOG_STATUS(Status_FragmentInfoError(
-        "Cannot get non-empty domain var; Invalid fragment index"));
-  if (dim_name == nullptr)
-    return LOG_STATUS(
-        Status_FragmentInfoError("Cannot get non-empty domain var; Dimension "
-                                 "name argument cannot be null"));
+  if (fid >= fragment_num()) {
+    throw FragmentInfoException(
+        "Cannot get non-empty domain var; Invalid fragment index");
+  }
+  if (dim_name == nullptr) {
+    throw FragmentInfoException(
+        "Cannot get non-empty domain var; Dimension "
+        "name argument cannot be null");
+  }
 
   auto meta = single_fragment_info_vec_[fid].meta();
   const auto& array_schema = meta->array_schema();
@@ -448,11 +452,9 @@ Status FragmentInfo::get_non_empty_domain_var(
 
   // Dimension name not found
   if (did == dim_num) {
-    auto msg =
-        std::string(
-            "Cannot get non-empty domain var; Invalid dimension name '") +
-        dim_name + "'";
-    return LOG_STATUS(Status_FragmentInfoError(msg));
+    throw FragmentInfoException(
+        "Cannot get non-empty domain var; Invalid dimension name '" +
+        std::string(dim_name) + "'");
   }
 
   return get_non_empty_domain_var(fid, did, start, end);
@@ -460,13 +462,15 @@ Status FragmentInfo::get_non_empty_domain_var(
 
 Status FragmentInfo::get_mbr_num(uint32_t fid, uint64_t* mbr_num) {
   ensure_loaded();
-  if (mbr_num == nullptr)
-    return LOG_STATUS(Status_FragmentInfoError(
-        "Cannot get fragment URI; MBR number argument cannot be null"));
+  if (mbr_num == nullptr) {
+    throw FragmentInfoException(
+        "Cannot get fragment URI; MBR number argument cannot be null");
+  }
 
-  if (fid >= fragment_num())
-    return LOG_STATUS(Status_FragmentInfoError(
-        "Cannot get fragment URI; Invalid fragment index"));
+  if (fid >= fragment_num()) {
+    throw FragmentInfoException(
+        "Cannot get fragment URI; Invalid fragment index");
+  }
 
   if (!single_fragment_info_vec_[fid].sparse()) {
     *mbr_num = 0;
@@ -474,7 +478,7 @@ Status FragmentInfo::get_mbr_num(uint32_t fid, uint64_t* mbr_num) {
   }
 
   auto meta = single_fragment_info_vec_[fid].meta();
-  meta->load_rtree(enc_key_);
+  meta->loaded_metadata()->load_rtree(enc_key_);
   *mbr_num = meta->mbrs().size();
 
   return Status::Ok();
@@ -483,36 +487,35 @@ Status FragmentInfo::get_mbr_num(uint32_t fid, uint64_t* mbr_num) {
 Status FragmentInfo::get_mbr(
     uint32_t fid, uint32_t mid, uint32_t did, void* mbr) {
   ensure_loaded();
-  if (mbr == nullptr)
-    return LOG_STATUS(Status_FragmentInfoError(
-        "Cannot get MBR; mbr argument cannot be null"));
+  if (mbr == nullptr) {
+    throw FragmentInfoException("Cannot get MBR; mbr argument cannot be null");
+  }
 
-  if (fid >= fragment_num())
-    return LOG_STATUS(
-        Status_FragmentInfoError("Cannot get MBR; Invalid fragment index"));
+  if (fid >= fragment_num()) {
+    throw FragmentInfoException("Cannot get MBR; Invalid fragment index");
+  }
 
-  if (!single_fragment_info_vec_[fid].sparse())
-    return LOG_STATUS(
-        Status_FragmentInfoError("Cannot get MBR; Fragment is not sparse"));
+  if (!single_fragment_info_vec_[fid].sparse()) {
+    throw FragmentInfoException("Cannot get MBR; Fragment is not sparse");
+  }
 
   auto meta = single_fragment_info_vec_[fid].meta();
-  meta->load_rtree(enc_key_);
+  meta->loaded_metadata()->load_rtree(enc_key_);
   const auto& mbrs = meta->mbrs();
 
-  if (mid >= mbrs.size())
-    return LOG_STATUS(
-        Status_FragmentInfoError("Cannot get MBR; Invalid MBR index"));
+  if (mid >= mbrs.size()) {
+    throw FragmentInfoException("Cannot get MBR; Invalid MBR index");
+  }
 
   const auto& minimum_bounding_rectangle = mbrs[mid];
-  if (did >= minimum_bounding_rectangle.size())
-    return LOG_STATUS(
-        Status_FragmentInfoError("Cannot get MBR; Invalid dimension index"));
+  if (did >= minimum_bounding_rectangle.size()) {
+    throw FragmentInfoException("Cannot get MBR; Invalid dimension index");
+  }
 
-  if (minimum_bounding_rectangle[did].var_size())
-    return LOG_STATUS(Status_FragmentInfoError(
-        "Cannot get MBR; Dimension is variable-sized"));
+  if (minimum_bounding_rectangle[did].var_size()) {
+    throw FragmentInfoException("Cannot get MBR; Dimension is variable-sized");
+  }
 
-  assert(!minimum_bounding_rectangle[did].empty());
   std::memcpy(
       mbr,
       minimum_bounding_rectangle[did].data(),
@@ -524,12 +527,13 @@ Status FragmentInfo::get_mbr(
 Status FragmentInfo::get_mbr(
     uint32_t fid, uint32_t mid, const char* dim_name, void* mbr) {
   ensure_loaded();
-  if (fid >= fragment_num())
-    return LOG_STATUS(
-        Status_FragmentInfoError("Cannot get MBR; Invalid fragment index"));
-  if (dim_name == nullptr)
-    return LOG_STATUS(Status_FragmentInfoError(
-        "Cannot get non-empty domain; Dimension name argument cannot be null"));
+  if (fid >= fragment_num()) {
+    throw FragmentInfoException("Cannot get MBR; Invalid fragment index");
+  }
+  if (dim_name == nullptr) {
+    throw FragmentInfoException(
+        "Cannot get non-empty domain; Dimension name argument cannot be null");
+  }
 
   auto meta = single_fragment_info_vec_[fid].meta();
   const auto& array_schema = meta->array_schema();
@@ -543,10 +547,9 @@ Status FragmentInfo::get_mbr(
 
   // Dimension name not found
   if (did == dim_num) {
-    auto msg =
-        std::string("Cannot get non-empty domain; Invalid dimension name '") +
-        dim_name + "'";
-    return LOG_STATUS(Status_FragmentInfoError(msg));
+    throw FragmentInfoException(
+        "Cannot get non-empty domain; Invalid dimension name '" +
+        std::string(dim_name) + "'");
   }
 
   return get_mbr(fid, mid, did, mbr);
@@ -559,43 +562,44 @@ Status FragmentInfo::get_mbr_var_size(
     uint64_t* start_size,
     uint64_t* end_size) {
   ensure_loaded();
-  if (start_size == nullptr)
-    return LOG_STATUS(
-        Status_FragmentInfoError("Cannot get MBR var size; Start "
-                                 "size argument cannot be null"));
+  if (start_size == nullptr) {
+    throw FragmentInfoException(
+        "Cannot get MBR var size; Start size argument cannot be null");
+  }
 
-  if (end_size == nullptr)
-    return LOG_STATUS(
-        Status_FragmentInfoError("Cannot get MBR var size; End "
-                                 "size argument cannot be null"));
+  if (end_size == nullptr) {
+    throw FragmentInfoException(
+        "Cannot get MBR var size; End size argument cannot be null");
+  }
 
-  if (fid >= fragment_num())
-    return LOG_STATUS(Status_FragmentInfoError(
-        "Cannot get MBR var size; Invalid fragment index"));
+  if (fid >= fragment_num()) {
+    throw FragmentInfoException(
+        "Cannot get MBR var size; Invalid fragment index");
+  }
 
-  if (!single_fragment_info_vec_[fid].sparse())
-    return LOG_STATUS(
-        Status_FragmentInfoError("Cannot get MBR; Fragment is not sparse"));
+  if (!single_fragment_info_vec_[fid].sparse()) {
+    throw FragmentInfoException("Cannot get MBR; Fragment is not sparse");
+  }
 
   auto meta = single_fragment_info_vec_[fid].meta();
-  meta->load_rtree(enc_key_);
+  meta->loaded_metadata()->load_rtree(enc_key_);
   const auto& mbrs = meta->mbrs();
 
-  if (mid >= mbrs.size())
-    return LOG_STATUS(
-        Status_FragmentInfoError("Cannot get MBR; Invalid mbr index"));
+  if (mid >= mbrs.size()) {
+    throw FragmentInfoException("Cannot get MBR; Invalid mbr index");
+  }
 
   const auto& minimum_bounding_rectangle = mbrs[mid];
+  if (did >= minimum_bounding_rectangle.size()) {
+    throw FragmentInfoException(
+        "Cannot get MBR var size; Invalid dimension index");
+  }
 
-  if (did >= minimum_bounding_rectangle.size())
-    return LOG_STATUS(Status_FragmentInfoError(
-        "Cannot get MBR var size; Invalid dimension index"));
+  if (!minimum_bounding_rectangle[did].var_size()) {
+    throw FragmentInfoException(
+        "Cannot get MBR var size; Dimension is fixed sized");
+  }
 
-  if (!minimum_bounding_rectangle[did].var_size())
-    return LOG_STATUS(Status_FragmentInfoError(
-        "Cannot get MBR var size; Dimension is fixed sized"));
-
-  assert(!minimum_bounding_rectangle[did].empty());
   *start_size = minimum_bounding_rectangle[did].start_size();
   *end_size = minimum_bounding_rectangle[did].end_size();
 
@@ -609,13 +613,15 @@ Status FragmentInfo::get_mbr_var_size(
     uint64_t* start_size,
     uint64_t* end_size) {
   ensure_loaded();
-  if (fid >= fragment_num())
-    return LOG_STATUS(Status_FragmentInfoError(
-        "Cannot get MBR var size; Invalid fragment index"));
-  if (dim_name == nullptr)
-    return LOG_STATUS(
-        Status_FragmentInfoError("Cannot get MBR var size; "
-                                 "Dimension name argument cannot be null"));
+  if (fid >= fragment_num()) {
+    throw FragmentInfoException(
+        "Cannot get MBR var size; Invalid fragment index");
+  }
+  if (dim_name == nullptr) {
+    throw FragmentInfoException(
+        "Cannot get MBR var size; "
+        "Dimension name argument cannot be null");
+  }
 
   auto meta = single_fragment_info_vec_[fid].meta();
   const auto& array_schema = meta->array_schema();
@@ -629,10 +635,9 @@ Status FragmentInfo::get_mbr_var_size(
 
   // Dimension name not found
   if (did == dim_num) {
-    auto msg =
-        std::string("Cannot get MBR var size; Invalid dimension name '") +
-        dim_name + "'";
-    return LOG_STATUS(Status_FragmentInfoError(msg));
+    throw FragmentInfoException(
+        "Cannot get MBR var size; Invalid dimension name '" +
+        std::string(dim_name) + "'");
   }
 
   return get_mbr_var_size(fid, mid, did, start_size, end_size);
@@ -641,41 +646,41 @@ Status FragmentInfo::get_mbr_var_size(
 Status FragmentInfo::get_mbr_var(
     uint32_t fid, uint32_t mid, uint32_t did, void* start, void* end) {
   ensure_loaded();
-  if (start == nullptr)
-    return LOG_STATUS(Status_FragmentInfoError(
-        "Cannot get MBR var; Start argument cannot be null"));
+  if (start == nullptr) {
+    throw FragmentInfoException(
+        "Cannot get MBR var; Start argument cannot be null");
+  }
 
-  if (end == nullptr)
-    return LOG_STATUS(Status_FragmentInfoError(
-        "Cannot get MBR var; End argument cannot be null"));
+  if (end == nullptr) {
+    throw FragmentInfoException(
+        "Cannot get MBR var; End argument cannot be null");
+  }
 
-  if (fid >= fragment_num())
-    return LOG_STATUS(
-        Status_FragmentInfoError("Cannot get MBR var; Invalid fragment index"));
+  if (fid >= fragment_num()) {
+    throw FragmentInfoException("Cannot get MBR var; Invalid fragment index");
+  }
 
-  if (!single_fragment_info_vec_[fid].sparse())
-    return LOG_STATUS(
-        Status_FragmentInfoError("Cannot get MBR var; Fragment is not sparse"));
+  if (!single_fragment_info_vec_[fid].sparse()) {
+    throw FragmentInfoException("Cannot get MBR var; Fragment is not sparse");
+  }
 
   auto meta = single_fragment_info_vec_[fid].meta();
-  meta->load_rtree(enc_key_);
+  meta->loaded_metadata()->load_rtree(enc_key_);
   const auto& mbrs = meta->mbrs();
 
-  if (mid >= mbrs.size())
-    return LOG_STATUS(
-        Status_FragmentInfoError("Cannot get MBR var; Invalid mbr index"));
+  if (mid >= mbrs.size()) {
+    throw FragmentInfoException("Cannot get MBR var; Invalid mbr index");
+  }
 
   const auto& minimum_bounding_rectangle = mbrs[mid];
+  if (did >= minimum_bounding_rectangle.size()) {
+    throw FragmentInfoException("Cannot get MBR var; Invalid dimension index");
+  }
 
-  if (did >= minimum_bounding_rectangle.size())
-    return LOG_STATUS(Status_FragmentInfoError(
-        "Cannot get MBR var; Invalid dimension index"));
+  if (!minimum_bounding_rectangle[did].var_size()) {
+    throw FragmentInfoException("Cannot get MBR var; Dimension is fixed-sized");
+  }
 
-  if (!minimum_bounding_rectangle[did].var_size())
-    return LOG_STATUS(Status_FragmentInfoError(
-        "Cannot get MBR var; Dimension is fixed-sized"));
-
-  assert(!minimum_bounding_rectangle[did].empty());
   auto start_str = minimum_bounding_rectangle[did].start_str();
   std::memcpy(start, start_str.data(), start_str.size());
   auto end_str = minimum_bounding_rectangle[did].end_str();
@@ -687,13 +692,14 @@ Status FragmentInfo::get_mbr_var(
 Status FragmentInfo::get_mbr_var(
     uint32_t fid, uint32_t mid, const char* dim_name, void* start, void* end) {
   ensure_loaded();
-  if (fid >= fragment_num())
-    return LOG_STATUS(
-        Status_FragmentInfoError("Cannot get MBR var; Invalid fragment index"));
-  if (dim_name == nullptr)
-    return LOG_STATUS(
-        Status_FragmentInfoError("Cannot get MBR var; Dimension "
-                                 "name argument cannot be null"));
+  if (fid >= fragment_num()) {
+    throw FragmentInfoException("Cannot get MBR var; Invalid fragment index");
+  }
+  if (dim_name == nullptr) {
+    throw FragmentInfoException(
+        "Cannot get MBR var; Dimension "
+        "name argument cannot be null");
+  }
 
   auto meta = single_fragment_info_vec_[fid].meta();
   const auto& array_schema = meta->array_schema();
@@ -707,11 +713,9 @@ Status FragmentInfo::get_mbr_var(
 
   // Dimension name not found
   if (did == dim_num) {
-    auto msg =
-        std::string(
-            "Cannot get non-empty domain var; Invalid dimension name '") +
-        dim_name + "'";
-    return LOG_STATUS(Status_FragmentInfoError(msg));
+    throw FragmentInfoException(
+        "Cannot get non-empty domain var; Invalid dimension name '" +
+        std::string(dim_name) + "'");
   }
 
   return get_mbr_var(fid, mid, did, start, end);
@@ -719,13 +723,14 @@ Status FragmentInfo::get_mbr_var(
 
 Status FragmentInfo::get_version(uint32_t fid, uint32_t* version) const {
   ensure_loaded();
-  if (version == nullptr)
-    return LOG_STATUS(Status_FragmentInfoError(
-        "Cannot get version; Version argument cannot be null"));
+  if (version == nullptr) {
+    throw FragmentInfoException(
+        "Cannot get version; Version argument cannot be null");
+  }
 
-  if (fid >= fragment_num())
-    return LOG_STATUS(
-        Status_FragmentInfoError("Cannot get version; Invalid fragment index"));
+  if (fid >= fragment_num()) {
+    throw FragmentInfoException("Cannot get version; Invalid fragment index");
+  }
 
   *version = single_fragment_info_vec_[fid].format_version();
 
@@ -735,10 +740,8 @@ Status FragmentInfo::get_version(uint32_t fid, uint32_t* version) const {
 shared_ptr<ArraySchema> FragmentInfo::get_array_schema(uint32_t fid) {
   ensure_loaded();
   if (fid >= fragment_num()) {
-    auto st = Status_FragmentInfoError(
+    throw FragmentInfoException(
         "Cannot get array schema; Invalid fragment index");
-    LOG_STATUS_NO_RETURN_VALUE(st);
-    throw_if_not_ok(st);
   }
   URI schema_uri;
   uint32_t version = single_fragment_info_vec_[fid].format_version();
@@ -759,13 +762,15 @@ shared_ptr<ArraySchema> FragmentInfo::get_array_schema(uint32_t fid) {
 Status FragmentInfo::get_array_schema_name(
     uint32_t fid, const char** schema_name) {
   ensure_loaded();
-  if (schema_name == nullptr)
-    return LOG_STATUS(Status_FragmentInfoError(
-        "Cannot get array schema URI; schema name argument cannot be null"));
+  if (schema_name == nullptr) {
+    throw FragmentInfoException(
+        "Cannot get array schema URI; schema name argument cannot be null");
+  }
 
-  if (fid >= fragment_num())
-    return LOG_STATUS(Status_FragmentInfoError(
-        "Cannot get array schema name; Invalid fragment index"));
+  if (fid >= fragment_num()) {
+    throw FragmentInfoException(
+        "Cannot get array schema name; Invalid fragment index");
+  }
 
   uint32_t version = single_fragment_info_vec_[fid].format_version();
   if (version >= 10) {
@@ -780,15 +785,17 @@ Status FragmentInfo::get_array_schema_name(
 Status FragmentInfo::has_consolidated_metadata(
     uint32_t fid, int32_t* has) const {
   ensure_loaded();
-  if (has == nullptr)
-    return LOG_STATUS(
-        Status_FragmentInfoError("Cannot check if fragment has consolidated "
-                                 "metadata; Has argument cannot be null"));
+  if (has == nullptr) {
+    throw FragmentInfoException(
+        "Cannot check if fragment has consolidated "
+        "metadata; Has argument cannot be null");
+  }
 
-  if (fid >= fragment_num())
-    return LOG_STATUS(
-        Status_FragmentInfoError("Cannot check if fragment has consolidated "
-                                 "metadata; Invalid fragment index"));
+  if (fid >= fragment_num()) {
+    throw FragmentInfoException(
+        "Cannot check if fragment has consolidated "
+        "metadata; Invalid fragment index");
+  }
 
   *has = single_fragment_info_vec_[fid].has_consolidated_footer();
 
@@ -802,8 +809,8 @@ Status FragmentInfo::load() {
   if (array_uri_.is_tiledb()) {
     auto rest_client = resources_->rest_client();
     if (rest_client == nullptr) {
-      return LOG_STATUS(Status_ArrayError(
-          "Cannot load fragment info; remote array with no REST client."));
+      throw FragmentInfoException(
+          "Cannot load fragment info; remote array with no REST client.");
     }
 
     // Overriding this config parameter is necessary to enable Cloud to load
@@ -867,7 +874,7 @@ Status FragmentInfo::load(const ArrayDirectory& array_dir) {
 
   // Get fragment sizes
   std::vector<uint64_t> sizes(fragment_num, 0);
-  RETURN_NOT_OK(parallel_for(
+  throw_if_not_ok(parallel_for(
       &resources_->compute_tp(),
       0,
       fragment_num,
@@ -881,7 +888,7 @@ Status FragmentInfo::load(const ArrayDirectory& array_dir) {
         }
 
         if (preload_rtrees & !meta->dense()) {
-          meta->load_rtree(enc_key_);
+          meta->loaded_metadata()->load_rtree(enc_key_);
         }
 
         return Status::Ok();
@@ -905,8 +912,10 @@ Status FragmentInfo::load(const ArrayDirectory& array_dir) {
 
       // compute expanded non-empty domain (only for dense fragments)
       auto expanded_non_empty_domain = non_empty_domain;
-      if (!sparse)
-        array_schema->domain().expand_to_tiles(&expanded_non_empty_domain);
+      if (!sparse) {
+        array_schema->current_domain().expand_to_tiles(
+            array_schema->domain(), expanded_non_empty_domain);
+      }
 
       // Push new fragment info
       single_fragment_info_vec_.emplace_back(SingleFragmentInfo(
@@ -936,7 +945,7 @@ Status FragmentInfo::load(const ArrayDirectory& array_dir) {
 
 void FragmentInfo::ensure_loaded() const {
   if (!loaded_) {
-    throw Status_FragmentInfoError("Fragment info has not been loaded.");
+    throw FragmentInfoException("Fragment info has not been loaded.");
   }
 }
 
@@ -964,9 +973,10 @@ load_consolidated_fragment_meta(
       resources.stats().start_timer("sm_read_load_consolidated_frag_meta");
 
   // No consolidated fragment metadata file
-  if (uri.to_string().empty())
-    throw StatusException(Status_FragmentInfoError(
-        "Cannot load consolidated fragment metadata; URI is empty."));
+  if (uri.to_string().empty()) {
+    throw FragmentInfoException(
+        "Cannot load consolidated fragment metadata; URI is empty.");
+  }
 
   auto tile = GenericTileIO::load(resources, uri, 0, enc_key, memory_tracker);
 
@@ -1146,8 +1156,10 @@ tuple<Status, optional<SingleFragmentInfo>> FragmentInfo::load(
   // (only for dense fragments)
   const auto& non_empty_domain = meta->non_empty_domain();
   auto expanded_non_empty_domain = non_empty_domain;
-  if (!sparse)
-    meta->array_schema()->domain().expand_to_tiles(&expanded_non_empty_domain);
+  if (!sparse) {
+    meta->array_schema()->current_domain().expand_to_tiles(
+        meta->array_schema()->domain(), expanded_non_empty_domain);
+  }
 
   // Set fragment info
   ret = SingleFragmentInfo(
@@ -1197,3 +1209,31 @@ Status FragmentInfo::replace(
 }
 
 }  // namespace tiledb::sm
+
+std::ostream& operator<<(
+    std::ostream& os, const tiledb::sm::FragmentInfo& fragment_info) {
+  auto fragment_num = fragment_info.fragment_num();
+
+  os << "- Fragment num: " << fragment_num << "\n";
+  os << "- Unconsolidated metadata num: "
+     << fragment_info.unconsolidated_metadata_num() << "\n";
+
+  auto to_vacuum = fragment_info.to_vacuum();
+  os << "- To vacuum num: " << to_vacuum.size() << "\n";
+
+  if (!to_vacuum.empty()) {
+    os << "- To vacuum URIs:\n";
+    for (const auto& v : to_vacuum)
+      os << "  > " << v.c_str() << "\n";
+  }
+
+  auto single_fragment_info_vec = fragment_info.single_fragment_info_vec();
+  for (uint32_t fid = 0; fid < fragment_num; ++fid) {
+    auto meta = single_fragment_info_vec[fid].meta();
+    auto dim_types = meta->dim_types();
+    os << "- Fragment #" << fid + 1 << ":\n";
+    single_fragment_info_vec[fid].dump_single_fragment_info(os, dim_types);
+  }
+
+  return os;
+}
