@@ -5,7 +5,7 @@
  *
  * The MIT License
  *
- * @copyright Copyright (c) 2017-2023 TileDB, Inc.
+ * @copyright Copyright (c) 2017-2024 TileDB, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -49,27 +49,18 @@
 #include "tiledb/sm/query/query_buffer.h"
 #include "tiledb/sm/query/query_condition.h"
 #include "tiledb/sm/query/query_remote_buffer_storage.h"
+#include "tiledb/sm/query/query_state.h"
 #include "tiledb/sm/query/readers/aggregators/iaggregator.h"
 #include "tiledb/sm/query/readers/aggregators/query_channel.h"
 #include "tiledb/sm/query/update_value.h"
 #include "tiledb/sm/query/validity_vector.h"
 #include "tiledb/sm/rest/rest_client.h"
-#include "tiledb/sm/storage_manager/storage_manager_declaration.h"
+#include "tiledb/sm/storage_manager/cancellation_source.h"
 #include "tiledb/sm/subarray/subarray.h"
 
 using namespace tiledb::common;
 
-namespace tiledb {
-namespace sm {
-
-/** Class for query status exceptions. */
-class QueryException : public StatusException {
- public:
-  explicit QueryException(const std::string& msg)
-      : StatusException("Query", msg) {
-  }
-};
-using QueryStatusException = QueryException;
+namespace tiledb::sm {
 
 class Array;
 class ArrayDimensionLabelQueries;
@@ -148,18 +139,34 @@ class Query {
    * case the query will be used as writes and the given URI should be used
    * for the name of the new fragment to be created.
    *
-   * @note Array must be a properly opened array.
+   * @section Maturity
    *
+   * This is a transitional constructor. There is still a `StorageManager`
+   * argument, and there is also a vestige of it with the `CancellationSource`
+   * argument. These argument now only pertain to job control of query with
+   * respect to its context. Once this facility is rewritten, these constructor
+   * argument may be dropped.
+   *
+   * @pre Array must be a properly opened array.
+   *
+   * @param resources The context resources.
+   * @param cancellation_source A source of external cancellation events
+   * @param storage_manager Storage manager object.
    * @param array The array that is being queried.
    * @param fragment_uri The full URI for the new fragment. Only used for
    * writes.
    * @param fragment_base_uri Optional base name for new fragment. Only used for
    *     writes and only if fragment_uri is empty.
+   * @param memory_budget Total memory budget. If set to nullopt, the value
+   *     will be obtained from the sm.mem.total_budget config option.
    */
   Query(
+      ContextResources& resources,
+      CancellationSource cancellation_source,
       StorageManager* storage_manager,
       shared_ptr<Array> array,
-      optional<std::string> fragment_name = nullopt);
+      optional<std::string> fragment_name = nullopt,
+      optional<uint64_t> memory_budget = nullopt);
 
   /** Destructor. */
   virtual ~Query();
@@ -179,10 +186,10 @@ class Query {
    * - The field is fixed-sized
    *
    * @param origin The name of the operation that this validation is a part of
-   * @param name The name of a field
+   * @param field_name The name of a field
    */
   void field_require_array_fixed(
-      const std::string_view origin, const char* name);
+      const std::string_view origin, std::string_view field_name);
 
   /**
    * Require that a name be that of a variable-size field from the source array.
@@ -192,10 +199,10 @@ class Query {
    * - The field is variable-sized
    *
    * @param origin The name of the operation that this validation is a part of
-   * @param name The name of a field
+   * @param field_name The name of a field
    */
   void field_require_array_variable(
-      const std::string_view origin, const char* name);
+      const std::string_view origin, std::string_view field_name);
 
   /**
    * Require that a field be a nullable field from the source array.
@@ -205,10 +212,10 @@ class Query {
    * - The attribute is nullable
    *
    * @param origin The name of the operation that this validation is a part of
-   * @param name The name of a field
+   * @param field_name The name of a field
    */
   void field_require_array_nullable(
-      const std::string_view origin, const char* name);
+      const std::string_view origin, std::string_view field_name);
 
   /**
    * Require that a field be a nonnull field from the source array.
@@ -218,44 +225,46 @@ class Query {
    * - The field is not nullable
    *
    * @param origin The name of the operation that this validation is a part of.
-   * @param name The name of a field
+   * @param field_name The name of a field
    */
   void field_require_array_nonnull(
-      const std::string_view origin, const char* name);
+      const std::string_view origin, std::string_view field_name);
 
   /**
    * Gets the estimated result size (in bytes) for the input fixed-sized
    * attribute/dimension.
    */
-  FieldDataSize get_est_result_size_fixed_nonnull(const char* name);
+  FieldDataSize get_est_result_size_fixed_nonnull(std::string_view field_name);
 
   /**
    * Gets the estimated result size (in bytes) for the input var-sized
    * attribute/dimension.
    */
-  FieldDataSize get_est_result_size_variable_nonnull(const char* name);
+  FieldDataSize get_est_result_size_variable_nonnull(
+      std::string_view field_name);
 
   /**
    * Gets the estimated result size (in bytes) for the input fixed-sized,
    * nullable attribute.
    */
-  FieldDataSize get_est_result_size_fixed_nullable(const char* name);
+  FieldDataSize get_est_result_size_fixed_nullable(std::string_view field_name);
 
   /**
    * Gets the estimated result size (in bytes) for the input var-sized,
    * nullable attribute.
    */
-  FieldDataSize get_est_result_size_variable_nullable(const char* name);
+  FieldDataSize get_est_result_size_variable_nullable(
+      std::string_view field_name);
 
  private:
   /**
    * Common part of all `est_result_size_*` functions, called after argument
    * validation.
    *
-   * @param name The name of a field
+   * @param field_name The name of a field
    * @return estimated result size
    */
-  FieldDataSize internal_est_result_size(const char* name);
+  FieldDataSize internal_est_result_size(std::string_view field_name);
 
  public:
   /** Retrieves the number of written fragments. */
@@ -310,12 +319,17 @@ class Query {
   QueryBuffer buffer(const std::string& name) const;
 
   /**
-   * Marks a query that has not yet been started as failed. This should not be
-   * called asynchronously to cancel an in-progress query; for that use the
-   * parent StorageManager's cancellation mechanism.
-   * @return Status
+   * Cancel a query.
+   *
+   * This does not immediately halt processing of a query, but does so at the
+   * first point where it checks in to see if it should continue processing.
    */
-  Status cancel();
+  void cancel();
+
+  /**
+   * Predicate function whether the query has been cancelled.
+   */
+  bool cancelled();
 
   /**
    * Finalizes the query, flushing all internal state.
@@ -701,14 +715,6 @@ class Query {
   /** Submits the query to the storage manager. */
   Status submit();
 
-  /**
-   * Submits the query to the storage manager. The query will be
-   * processed asynchronously (i.e., in a non-blocking manner).
-   * Once the query is completed, the input callback function will
-   * be executed using the input callback data.
-   */
-  Status submit_async(std::function<void(void*)> callback, void* callback_data);
-
   /** Returns the query status. */
   QueryStatus status() const;
 
@@ -866,14 +872,14 @@ class Query {
       }
       return;
     }
-    throw QueryStatusException(
+    throw QueryException(
         "We currently only support a default channel for queries");
   }
 
   /**
    * Returns true if the query has any aggregates on any channels
    */
-  bool has_aggregates() {
+  bool has_aggregates() const {
     // We only need to check the default channel for now
     return !default_channel_aggregates_.empty();
   }
@@ -901,13 +907,13 @@ class Query {
 
   inline std::shared_ptr<QueryChannel> aggegate_channel() {
     if (!has_aggregates()) {
-      throw QueryStatusException("Aggregate channel does not exist");
+      throw QueryException("Aggregate channel does not exist");
     }
     return aggregate_channel_;
   }
 
   /**
-   * Returns the REST client configured in the storage manager associated to
+   * Returns the REST client configured in the context resources associated to
    * this query
    */
   RestClient* rest_client() const;
@@ -917,9 +923,26 @@ class Query {
   /*         PRIVATE ATTRIBUTES        */
   /* ********************************* */
 
+  /** Resource used for operations. */
+  ContextResources& resources_;
+
+  /** The class stats. */
+  stats::Stats* stats_;
+
+  /** The class logger. */
+  shared_ptr<Logger> logger_;
+
   /** The query memory tracker. */
   shared_ptr<MemoryTracker> query_memory_tracker_;
 
+  /**
+   * The state machine for processing local queries.
+   *
+   * At present this class combines both local and remote queries. This member
+   * variable is essentially unused for remote queries.
+   */
+  LocalQueryStateMachine local_state_machine_{LocalQueryState::uninitialized};
+
   /** A smart pointer to the array the query is associated with.
    * Ensures that the Array object exists as long as the Query object exists. */
   shared_ptr<Array> array_shared_;
@@ -957,18 +980,18 @@ class Query {
   /** The query status. */
   QueryStatus status_;
 
+  /**
+   * The cancellation source. This will be the last vestige of the storage
+   * manager.
+   */
+  CancellationSource cancellation_source_;
+
   /** The storage manager. */
   StorageManager* storage_manager_;
 
   /** The query strategy. */
   tdb_unique_ptr<IQueryStrategy> strategy_;
 
-  /** The class stats. */
-  stats::Stats* stats_;
-
-  /** The class logger. */
-  shared_ptr<Logger> logger_;
-
   /** UID of the logger instance */
   inline static std::atomic<uint64_t> logger_id_ = 0;
 
@@ -1087,6 +1110,12 @@ class Query {
    */
   uint64_t fragment_size_;
 
+  /**
+   * Memory budget. If set to nullopt, the value will be obtained from the
+   * sm.mem.total_budget config option.
+   */
+  optional<uint64_t> memory_budget_;
+
   /** Already written buffers. */
   std::unordered_set<std::string> written_buffers_;
 
@@ -1120,6 +1149,15 @@ class Query {
   /*           PRIVATE METHODS         */
   /* ********************************* */
 
+  /**
+   * Return the layout that will be used to execute the query.
+   *
+   * This is usually set by the user but can be overridden by TileDB in cases
+   * where the data order would not have an observable difference, like queries
+   * with only aggregates.
+   */
+  Layout effective_layout() const;
+
   /**
    * Create the strategy.
    *
@@ -1167,7 +1205,6 @@ class Query {
   void copy_aggregates_data_to_user_buffer();
 };
 
-}  // namespace sm
-}  // namespace tiledb
+}  // namespace tiledb::sm
 
 #endif  // TILEDB_QUERY_H
