@@ -38,7 +38,6 @@
 #include "tiledb/sm/enums/query_condition_op.h"
 #include "tiledb/sm/fragment/fragment_identifier.h"
 #include "tiledb/sm/fragment/fragment_metadata.h"
-#include "tiledb/sm/misc/utils.h"
 #include "tiledb/sm/query/readers/result_cell_slab.h"
 
 #include <algorithm>
@@ -646,8 +645,10 @@ void QueryCondition::apply_ast_node(
       // delete condition was already processed, GT condition is always true.
       if (field_name == constants::delete_timestamps &&
           (!fragment_metadata[f]->has_delete_meta() ||
-           fragment_metadata[f]->get_processed_conditions_set().count(
-               condition_marker_) != 0)) {
+           fragment_metadata[f]
+                   ->loaded_metadata()
+                   ->get_processed_conditions_set()
+                   .count(condition_marker_) != 0)) {
         assert(Op == QueryConditionOp::GT);
         for (size_t c = starting_index; c < starting_index + length; ++c) {
           result_cell_bitmap[c] = 1;
