@@ -85,6 +85,30 @@ struct HeaderCbData {
 
   /** A pointer to the lock attached to the shared resource of the cache map */
   std::mutex* redirect_uri_map_lock;
+
+  /** True if the uri should be stored in URI cache map, false if not */
+  bool should_cache_redirect;
+};
+
+/**
+ * Wraps opaque user data to be invoked with a write callback.
+ */
+struct WriteCbState {
+  /** Default constructor. */
+  WriteCbState()
+      : reset(true)
+      , arg(NULL)
+      , skip_retries(false) {
+  }
+
+  /** True if this is the first write callback invoked in a request retry. */
+  bool reset;
+
+  /** The opaque user data to pass to the write callback. */
+  void* arg;
+
+  /** True if the internal curl retries should be skipped. */
+  bool skip_retries;
 };
 
 /**
@@ -128,7 +152,8 @@ class Curl {
       const Config* config,
       const std::unordered_map<std::string, std::string>& extra_headers,
       std::unordered_map<std::string, std::string>* res_headers,
-      std::mutex* res_mtx);
+      std::mutex* res_mtx,
+      bool should_cache_redirect = true);
 
   /**
    * Escapes the given URL.
@@ -154,7 +179,7 @@ class Curl {
       stats::Stats* stats,
       const std::string& url,
       SerializationType serialization_type,
-      const BufferList* data,
+      BufferList* data,
       Buffer* returned_data,
       const std::string& res_ns_uri);
 
@@ -174,7 +199,7 @@ class Curl {
       stats::Stats* stats,
       const std::string& url,
       SerializationType serialization_type,
-      const BufferList* data,
+      BufferList* data,
       Buffer* returned_data,
       const std::string& res_ns_uri);
 
@@ -194,7 +219,7 @@ class Curl {
       stats::Stats* stats,
       const std::string& url,
       SerializationType serialization_type,
-      const BufferList* data,
+      BufferList* data,
       Buffer* returned_data,
       const std::string& res_ns_uri);
 
@@ -247,7 +272,7 @@ class Curl {
       stats::Stats* stats,
       const std::string& url,
       SerializationType serialization_type,
-      const BufferList* data,
+      BufferList* data,
       Buffer* returned_data,
       PostResponseCb&& write_cb,
       const std::string& res_ns_uri);
@@ -264,7 +289,7 @@ class Curl {
    */
   Status patch_data_common(
       SerializationType serialization_type,
-      const BufferList* data,
+      BufferList* data,
       struct curl_slist** headers);
 
   /**
@@ -279,7 +304,7 @@ class Curl {
    */
   Status put_data_common(
       SerializationType serialization_type,
-      const BufferList* data,
+      BufferList* data,
       struct curl_slist** headers);
 
   /**
@@ -294,7 +319,7 @@ class Curl {
    */
   Status post_data_common(
       SerializationType serialization_type,
-      const BufferList* data,
+      BufferList* data,
       struct curl_slist** headers);
 
   /**
@@ -339,6 +364,15 @@ class Curl {
    */
   tuple<Status, optional<long>> last_http_status_code();
 
+  /**
+   * Checks if the request should be retried
+   *
+   * @param curl_code the curl code of the attempted request
+   * @param http_code http code to check
+   * @return retry or not
+   */
+  bool should_retry_request(CURLcode curl_code, long http_request) const;
+
  private:
   /**
    * A libcurl initializer instance. This should remain
@@ -386,6 +420,9 @@ class Curl {
   /** Max curl buffer size for received data. */
   uint64_t curl_buffer_size_;
 
+  /* Retry requests with curl errors */
+  bool retry_curl_errors_;
+
   /**
    * Populates the curl slist with authorization (token or username+password),
    * and any extra headers.
@@ -412,6 +449,7 @@ class Curl {
    * @param stats The stats instance to record into
    * @param url URL to fetch
    * @param curl_code Set to the return value of the curl call
+   * @param data Encoded data buffer for sending, if applicable to the request
    * @param returned_data Buffer that will store the response data
    * @return Status
    */
@@ -419,6 +457,7 @@ class Curl {
       stats::Stats* const stats,
       const char* url,
       CURLcode* curl_code,
+      BufferList* data,
       Buffer* returned_data) const;
 
   /**
@@ -428,6 +467,7 @@ class Curl {
    * @param stats The stats instance to record into
    * @param url URL to fetch
    * @param curl_code Set to the return value of the curl call
+   * @param data Encoded data buffer for sending, if applicable to the request
    * @param write_cb Callback to invoke as response data is received
    * @return Status
    */
@@ -435,6 +475,7 @@ class Curl {
       stats::Stats* stats,
       const char* url,
       CURLcode* curl_code,
+      BufferList* data,
       PostResponseCb&& write_cb) const;
 
   /**
@@ -443,6 +484,7 @@ class Curl {
    * @param stats The stats instance to record into
    * @param url URL to fetch
    * @param curl_code Set to the return value of the curl call
+   * @param data Encoded data buffer for sending, if applicable to the request
    * @param write_cb Callback to invoke as response data is received.
    * @param write_arg Opaque memory address passed to 'write_cb'.
    * @return Status
@@ -451,6 +493,7 @@ class Curl {
       stats::Stats* stats,
       const char* url,
       CURLcode* curl_code,
+      BufferList* data,
       size_t (*write_cb)(void*, size_t, size_t, void*),
       void* write_arg) const;
 
@@ -472,17 +515,16 @@ class Curl {
       const char* const url, const uint8_t retry_number) const;
 
   /**
-   * Common code shared between variants of 'make_curl_options_request'.
+   * Set needed options on the curl request
    *
-   * @param stats The stats instance to record into
    * @param url URL to fetch
-   * @param curl_code Set to the return value of the curl call
-   * @return Status
+   * @param write_cb Callback to invoke as response data is received.
+   * @param write_cb_state A data pointer to pass to the write callback.
    */
-  Status make_curl_request_options_common(
-      stats::Stats* const stats,
+  void set_curl_request_options(
       const char* const url,
-      CURLcode* const curl_code) const;
+      size_t (*write_cb)(void*, size_t, size_t, void*),
+      WriteCbState& write_cb_state) const;
 
   /**
    * Check the given curl code for errors, returning a TileDB error status if
@@ -510,10 +552,18 @@ class Curl {
   /**
    * Checks the curl http status code to see if it matches a list of http
    * requests to retry
-   * @param retry true if the http code matches the retry list
-   * @return Status
+   * @param http_code http code to check
+   * @return true if should be retried, false otherwise
    */
-  Status should_retry_based_on_http_status(bool* retry) const;
+  bool should_retry_based_on_http_status(long http_code) const;
+
+  /**
+   * Checks the curl return code to see if the request should be retried
+   *
+   * @param curl_code the curl code to check
+   * @return retry or not
+   */
+  bool should_retry_based_on_curl_code(CURLcode curl_code) const;
 };
 
 }  // namespace sm
